from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ..text.models import CharFieldDefinition
from ...models import FieldDefinitionManager


class _WebMeta:
    defined_field_category = _('Web')


class EmailFieldDefinition(CharFieldDefinition):
    class Meta(_WebMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.EmailField


class URLFieldDefinition(CharFieldDefinition):
    class Meta(_WebMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.URLField


class SlugFieldDefinition(CharFieldDefinition):
    class Meta(_WebMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.SlugField
        defined_field_description = _('slug')


class IPAddressFieldDefinition(CharFieldDefinition):
    class Meta(_WebMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.IPAddressField


protocol_help_text = _('Limits valid inputs to the specified protocol.')
unpack_ipv4_help_text = _('Unpacks IPv4 mapped addresses like '
                          '``::ffff::192.0.2.1`` to ``192.0.2.1``')

class GenericIPAddressFieldDefinition(CharFieldDefinition):
    PROTOCOL_BOTH = 'both'
    PROTOCOL_IPV4 = 'IPv4'
    PROTOCOL_IPV6 = 'IPv6'

    PROTOCOL_CHOICES = (
        (PROTOCOL_BOTH, _('both')),
        (PROTOCOL_IPV4, _('IPv4')),
        (PROTOCOL_IPV6, _('IPv6'))
    )

    protocol = fields.CharField(
        _('protocol'), max_length=4,
        choices=PROTOCOL_CHOICES, default=PROTOCOL_BOTH
    )
    unpack_ipv4 = fields.BooleanField(_('unpack ipv4'), default=False)

    objects = FieldDefinitionManager()

    class Meta(_WebMeta):
        app_label = 'mutant'
        defined_field_class = fields.GenericIPAddressField
        defined_field_options = ('protocol', 'unpack_ipv4',)
        defined_field_description = _('generic IP address')

    def clean(self):
        if self.unpack_ipv4 and self.procotol != 'both':
            msg = _("Can only be used when ``protocol`` is set to 'both'.")
            raise ValidationError({'unpack_ipv4': msg})



from __future__ import unicode_literals

from django.forms.forms import Form
from django.test.testcases import TestCase

from mutant.forms import FieldDefinitionTypeField
from mutant.models.field import FieldDefinition


class FieldDefinitionTypeFieldTest(TestCase):
    def test_invalid_fd(self):
        with self.assertRaisesMessage(TypeError,
                                      'is not a subclass of FieldDefinitionBase'):
            FieldDefinitionTypeField((FieldDefinitionTypeField,))

    def test_valid_fd(self):
        field = FieldDefinitionTypeField((FieldDefinition,))
        custom_field_ct = FieldDefinition.get_content_type()
        self.assertTrue(field.valid_value(FieldDefinition))
        self.assertTrue(field.valid_value(custom_field_ct))
        self.assertTrue(field.valid_value(custom_field_ct.pk))

    def test_form_validation(self):
        custom_field_ct = FieldDefinition.get_content_type()

        class CustomModelForm(Form):
            field_type = FieldDefinitionTypeField((FieldDefinition,))
        data = {'field_type': custom_field_ct.pk}
        form = CustomModelForm(data)
        self.assertTrue(form.is_valid())
        self.assertEqual(form.cleaned_data['field_type'], custom_field_ct)



from __future__ import unicode_literals

from django.db.models.deletion import CASCADE


def CASCADE_MARK_ORIGIN(collector, field, sub_objs, using):
    """
    Custom on_delete handler which sets  _cascade_deletion_origin on the _state
    of the  all relating objects that will deleted.
    We use this handler on ModelDefinitionAttribute.model_def, so when we delete
    a ModelDefinition we can skip field_definition_post_delete and
    base_definition_post_delete and avoid an incremental columns deletion before
    the entire table is dropped.
    """
    CASCADE(collector, field, sub_objs, using)
    if sub_objs:
        for obj in sub_objs:
            obj._state._cascade_deletion_origin = field.name



from __future__ import unicode_literals

import os
import re

from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.fields import CharField
from django.utils.translation import ugettext_lazy as _

from ...validators import validate_python_identifier


class DirectoryPathField(CharField):
    def to_python(self, value):
        value = super(DirectoryPathField, self).to_python(value)
        if value is None:
            return
        if not os.path.exists(value):
            raise ValidationError(_("Specified path doesn't exist"))
        elif not os.path.isdir(value):
            raise ValidationError(_("Specified path isn't a directory"))
        else:
            return value


class RegExpStringField(CharField):
    def to_python(self, value):
        value = super(RegExpStringField, self).to_python(value)
        if value is None:
            return
        try:
            re.compile(value)
        except Exception as e:
            raise ValidationError(_(e))
        else:
            return value


class PythonIdentifierField(CharField):
    __metaclass__ = models.SubfieldBase

    default_validators = [validate_python_identifier]
    description = _('Python identifier')

    def __init__(self, *args, **kwargs):
        defaults = {'max_length': 255}
        defaults.update(kwargs)
        super(PythonIdentifierField, self).__init__(*args, **defaults)

    def to_python(self, value):
        value = super(PythonIdentifierField, self).to_python(value)
        if value is not None:
            return str(value)



from __future__ import unicode_literals

from django.db.models import Q, signals
from django.db.models.fields.related import RelatedField
from django.dispatch.dispatcher import receiver

from ....db.models import MutableModel
from ....management import allow_syncdbs, perform_ddl
from ....models import ModelDefinition
from ....signals import mutable_class_prepared

from ..models import (ForeignKeyDefinition, ManyToManyFieldDefinition,
    OneToOneFieldDefinition)


_opts_related_cache_attrs = ('_related_objects_cache', '_related_objects_proxy_cache',
                             '_related_many_to_many_cache', '_name_map')

def clear_opts_related_cache(model_class):
    """
    Clear the specified model opts related cache
    """
    opts = model_class._meta
    for attr in _opts_related_cache_attrs:
        try:
            delattr(opts, attr)
        except AttributeError:
            pass


@receiver(mutable_class_prepared)
def mutable_model_prepared(signal, sender, definition, existing_model_class,
                           **kwargs):
    """
    Make sure all related model class are created and marked as dependency
    when a mutable model class is prepared
    """
    referenced_models = set()
    # Collect all model class the obsolete model class was referring to
    if existing_model_class:
        for field in existing_model_class._meta.local_fields:
            if isinstance(field, RelatedField):
                rel_to = field.rel.to
                if not isinstance(rel_to, basestring):
                    referenced_models.add(rel_to)
    # Add sender as a dependency of all mutable models it refers to
    for field in sender._meta.local_fields:
        if isinstance(field, RelatedField):
            rel_to = field.rel.to
            if not isinstance(rel_to, basestring):
                referenced_models.add(rel_to)
                if (issubclass(rel_to, MutableModel) and
                    rel_to._definition != sender._definition):
                    rel_to._dependencies.add(sender._definition)
    # Mark all model referring to this one as dependencies
    related_model_defs = ModelDefinition.objects.filter(
        Q(fielddefinitions__foreignkeydefinition__to=definition) |
        Q(fielddefinitions__manytomanyfielddefinition__to=definition)
    ).distinct()
    for model_def in related_model_defs:
        if model_def != definition:
            # Generate model class from definition and add it as a dependency
            sender._dependencies.add(model_def.model_class()._definition)
    # Clear the referenced models opts related cache
    for model_class in referenced_models:
        clear_opts_related_cache(model_class)


@receiver(signals.pre_delete, sender=ManyToManyFieldDefinition,
          dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_pre_delete')
def many_to_many_field_definition_pre_delete(sender, instance, **kwargs):
    model_class = instance.model_def.model_class()
    field = model_class._meta.get_field(str(instance.name))
    intermediary_table_name = field.rel.through._meta.db_table
    instance._state._m2m_deletion = (
        allow_syncdbs(model_class),
        intermediary_table_name
    )


@receiver(signals.post_delete, sender=ManyToManyFieldDefinition,
          dispatch_uid='mutant.contrib.related.management.many_to_many_field_definition_post_delete')
def many_to_many_field_definition_post_delete(sender, instance, **kwargs):
    syncdbs, intermediary_table_name = instance._state._m2m_deletion
    for db in syncdbs:
        db.delete_table(intermediary_table_name)
    del instance._state._m2m_deletion



from __future__ import unicode_literals

import warnings

import django
from django.db import models
from polymodels.managers import PolymorphicManager, PolymorphicQuerySet

from ...utils import choices_from_dict


class FieldDefinitionQuerySet(PolymorphicQuerySet):
    def create_with_default(self, default, **kwargs):
        obj = self.model(**kwargs)
        obj._state._creation_default_value = default
        self._for_write = True
        obj.save(force_insert=True, using=self.db)
        return obj


class FieldDefinitionManager(PolymorphicManager):
    def get_queryset(self):
        return FieldDefinitionQuerySet(self.model, using=self._db)

    if django.VERSION < (1, 8):
        def get_query_set(self):
            if django.VERSION >= (1, 6):
                warnings.warn(
                    "`FieldDefinitionManager.get_query_set` is deprecated, "
                    "use `get_queryset` instead.",
                    DeprecationWarning if django.VERSION >= (1, 7)
                        else PendingDeprecationWarning,
                    stacklevel=2
                )
            return FieldDefinitionManager.get_queryset(self)

    def get_by_natural_key(self, app_label, model, name):
        qs = self.select_subclasses()
        return qs.get(model_def__app_label=app_label,
                      model_def__model=model, name=name)

    def names(self):
        qs = self.get_queryset()
        return qs.order_by('name').values_list('name', flat=True)

    def create_with_default(self, default, **kwargs):
        qs = self.get_queryset()
        return qs.create_with_default(default, **kwargs)


class FieldDefinitionChoiceQuerySet(models.query.QuerySet):
    def as_choices(self):
        # Here we don't use .values() since it's raw output from the database
        # and values are not prepared correctly.
        choices = ({'group': choice.group,
                    'label': choice.label,
                    'value': choice.value}
                   for choice in self.only('group', 'value', 'label'))
        return choices_from_dict(choices)


class FieldDefinitionChoiceManager(models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        return FieldDefinitionChoiceQuerySet(self.model, using=self._db)

    if django.VERSION < (1, 8):
        def get_query_set(self):
            if django.VERSION >= (1, 6):
                warnings.warn(
                    "`FieldDefinitionChoiceManager.get_query_set` is"
                    "deprecated, use `get_queryset` instead.",
                    DeprecationWarning if django.VERSION >= (1, 7)
                        else PendingDeprecationWarning,
                    stacklevel=2
                )
            return FieldDefinitionChoiceManager.get_queryset(self)

    def as_choices(self):
        return self.get_queryset().as_choices()



from __future__ import unicode_literals

import sys
# TODO: Remove when support for Python 2.6 is dropped
if sys.version_info >= (2, 7):
    from unittest import expectedFailure
else:
    from django.utils.unittest import expectedFailure

from django.contrib.gis.geos import (GeometryCollection, LineString, Point,
    Polygon, MultiLineString, MultiPoint, MultiPolygon)
from django.utils.translation import ugettext_lazy as _

from mutant.models import BaseDefinition
from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import (GeoModel, GeometryFieldDefinition,
    GeometryCollectionFieldDefinition, LineStringFieldDefinition,
    PointFieldDefinition, PolygonFieldDefinition,
    MultiLineStringFieldDefinition, MultiPointFieldDefinition,
    MultiPolygonFieldDefinition)


class GeometryFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Geometry')

    def setUp(self):
        super(GeometryFieldDefinitionTestMixin, self).setUp()
        BaseDefinition.objects.create(model_def=self.model_def, base=GeoModel)

    @expectedFailure
    def test_create_with_default(self):
        """'Geometry field defaults are not correctly handled by South.'"""
        super(GeometryFieldDefinitionTestMixin, self).test_create_with_default()


class GeometryFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                  BaseModelDefinitionTestCase):
    field_definition_cls = GeometryFieldDefinition
    field_values = (
        LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
        Polygon(
            ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
        )
    )


class PointFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = PointFieldDefinition
    field_values = (Point(5, 23), Point(13, 37))


class LineStringFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                    BaseModelDefinitionTestCase):
    field_definition_cls = LineStringFieldDefinition
    field_values = (
        LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
        LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10))
    )


class PolygonFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                 BaseModelDefinitionTestCase):
    field_definition_cls = PolygonFieldDefinition
    field_values = (
        Polygon(
            ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0))
        ),
        Polygon(
            ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
        ),
    )


class MultiLineStringFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                         BaseModelDefinitionTestCase):
    field_definition_cls = MultiLineStringFieldDefinition
    field_values = (
        MultiLineString(
            LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
        ),
        MultiLineString(
            LineString((13, 7), (18, 50), (50, 50), (50, 27), (110, 0)),
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
            LineString((0, 0), (0, 50), (50, 50), (50, 0), (0, 0)),
        ),
    )


class MultiPointFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                    BaseModelDefinitionTestCase):
    field_definition_cls = MultiPointFieldDefinition
    field_values = (
        MultiPoint(Point(0, 0), Point(1, 1)),
        MultiPoint(Point(5, 23), Point(13, 37), Point(13, 58)),
    )


class MultiPolygonFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                      BaseModelDefinitionTestCase):
    field_definition_cls = MultiPolygonFieldDefinition
    field_values = (
        MultiPolygon(
            Polygon(
                ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0))
            ),
            Polygon(
                ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
            ),
        ),
        MultiPolygon(
            Polygon(
                ((0.0, 0.0), (0.0, 50.0), (50.0, 50.0), (50.0, 0.0), (0.0, 0.0))
            ),
            Polygon(
                ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
            ),
            Polygon(
                ((0.0, 0.0), (0.0, 50.0), (50.0, 51.0), (50.0, 45), (0.0, 0.0))
            ),
        ),
    )


class GeometryCollectionFieldDefinitionTest(GeometryFieldDefinitionTestMixin,
                                            BaseModelDefinitionTestCase):
    field_definition_cls = GeometryCollectionFieldDefinition
    field_values = (
        GeometryCollection(
            Point(0, 0),
            Polygon(
                ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
            ),
        ),
        GeometryCollection(
            LineString((1, 2), (3, 4), (5, 6), (7, 8), (9, 10)),
            Polygon(
                ((0.0, 0.0), (18, 50.0), (47.0, 55.0), (50.0, 0.0), (0.0, 0.0))
            ),
            Point(5, 23),
            Point(13, 37),
        ),
    )



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.db import connections, router, transaction
from django.db.utils import DEFAULT_DB_ALIAS, IntegrityError
from django.test.testcases import TestCase
from south.db import dbs as south_dbs

from ..models.model import ModelDefinition


class DDLTestCase(TestCase):
    """
    A class that behaves like `TestCase` if all connections support DDL
    transactions or like `TransactionTestCase` if it's not the case.
    """
    manual_transaction = False

    def connections_have_ddl_transactions(self):
        """
        Returns True if all implied connections have DDL transactions support.
        """
        db_names = connections if getattr(self, 'multi_db', False) else [DEFAULT_DB_ALIAS]
        return all(south_dbs[name].has_ddl_transactions for name in db_names)

    def _fixture_setup(self):
        if (not self.manual_transaction and
            self.connections_have_ddl_transactions()):
            return super(DDLTestCase, self)._fixture_setup()
        else:
            return super(TestCase, self)._fixture_setup()

    def _fixture_teardown(self):
        if (not self.manual_transaction and
            self.connections_have_ddl_transactions()):
            return super(DDLTestCase, self)._fixture_teardown()
        else:
            return super(TestCase, self)._fixture_teardown()


class ModelDefinitionDDLTestCase(DDLTestCase):
    def tearDown(self):
        if (self.manual_transaction or
            not self.connections_have_ddl_transactions()):
            # Remove all the extra tables since `TransactionTestCase` only
            # truncate data on teardown.
            ModelDefinition.objects.all().delete()
        ContentType.objects.clear_cache()


class FieldDefinitionTestMixin(object):
    field_definition_init_kwargs = {}
    field_values = ()

    def setUp(self):
        super(FieldDefinitionTestMixin, self).setUp()
        self.field = self.field_definition_cls._default_manager.create(
            model_def=self.model_def,
            name='field',
            **self.field_definition_init_kwargs
        )

    def get_field_value(self, instance, name='field'):
        return getattr(instance, name)

    def prepare_default_value(self, value):
        return value

    def test_field_default(self):
        default = self.prepare_default_value(self.field_values[0])
        field = self.field
        # Default value should be valid
        field.default = default
        field.full_clean()
        field.save()
        # Default value should be assigned correctly
        Model = self.model_def.model_class()
        instance = Model.objects.create()
        created_default = self.prepare_default_value(self.get_field_value(instance))
        self.assertEqual(created_default, default)

    def test_create_with_default(self):
        """
        Makes sure a field definition manager is attached to the model and
        `create_with_default` works correctly.
        """
        Model = self.model_def.model_class()
        field_value = self.field_values[0]
        instance = Model.objects.create(field=field_value)
        # Add the field with a default.
        create_default = self.prepare_default_value(field_value)
        options = dict(**self.field_definition_init_kwargs)
        options['default'] = create_default
        self.field_definition_cls._default_manager.create_with_default(
            model_def=self.model_def, name='field_created_with_default',
            **options
        )
        created_value = self.prepare_default_value(
            Model.objects.get(pk=instance.pk).field_created_with_default
        )
        self.assertEqual(created_value, create_default)

    def test_model_save(self):
        first_value, second_value = self.field_values
        # Assigning a value should work
        Model = self.model_def.model_class()
        instance = Model.objects.create(field=first_value)
        self.assertEqual(self.get_field_value(instance), first_value)
        # Assigning a new one should also work
        instance.field = second_value
        instance.save()
        instance = Model.objects.get()
        self.assertEqual(self.get_field_value(instance), second_value)

    def test_field_renaming(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        # Renaming a field should update its column name
        Model.objects.create(field=value)
        _, original_column_name = Model._meta.get_field('field').get_attname_column()
        self.field.name = 'renamed_field'
        self.field.save()
        _, new_column_name = Model._meta.get_field('renamed_field').get_attname_column()
        self.assertModelTablesColumnDoesntExists(Model, original_column_name)
        self.assertModelTablesColumnExists(Model, new_column_name)
        # Old data should be accessible by the new field name
        instance = Model.objects.get()
        self.assertEqual(self.get_field_value(instance, 'renamed_field'), value)
        # The old field shouldn't be accessible anymore
        msg = "'field' is an invalid keyword argument for this function"
        self.assertRaisesMessage(TypeError, msg, Model, field=value)
        # It should be possible to create objects using the new field name
        Model.objects.create(renamed_field=value)

    def test_field_deletion(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        Model.objects.create(field=value)
        # Deleting a field should delete the associated column
        _, field_column_name = Model._meta.get_field('field').get_attname_column()
        self.field.delete()
        self.assertModelTablesColumnDoesntExists(Model, field_column_name)
        # The deleted field shouldn't be accessible anymore
        msg = "'field' is an invalid keyword argument for this function"
        self.assertRaisesMessage(TypeError, msg, Model, field=value)

    def test_field_unique(self):
        value = self.field_values[0]
        Model = self.model_def.model_class()
        self.field.unique = True
        self.field.save()
        Model.objects.create(field=value)
        write_db = router.db_for_write(Model)
        sid = transaction.savepoint(using=write_db)
        try:
            Model.objects.create(field=value)
        except IntegrityError:
            pass
        else:
            self.fail("One shouldn't be able to save duplicate entries in a unique field")
        finally:
            transaction.savepoint_rollback(sid, using=write_db)

    def test_field_cloning(self):
        clone = self.field.clone()
        clone.name = 'field_clone'
        clone.model_def = self.model_def
        clone.save(force_insert=True)

    def test_field_definition_category(self):
        self.assertEqual(self.field_definition_cls.get_field_category(),
                         self.field_definition_category)



from __future__ import unicode_literals

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.translation import ugettext_lazy as _

from .. import logger
from ..utils import remove_from_app_cache


class MutableModel(models.Model):
    """
    Abstract class used to identify models that we're created by a definition
    """
    class Meta:
        abstract = True

    @classmethod
    def definition(cls):
        definition_cls, definition_pk = cls._definition
        return definition_cls.objects.get(pk=definition_pk)

    @classmethod
    def is_obsolete(cls):
        return cls._is_obsolete

    @classmethod
    def mark_as_obsolete(cls):
        remove_from_app_cache(cls)
        cls._is_obsolete = True
        logger.debug(
            "Marking model %s and it dependencies (%s) as obsolete.",
            cls, cls._dependencies
        )
        for definition_cls, definition_pk in cls._dependencies:
            try:
                definition = definition_cls.objects.get(pk=definition_pk)
            except definition_cls.DoesNotExist:
                pass
            else:
                definition.model_class().mark_as_obsolete()

    def clean(self):
        if self.is_obsolete():
            raise ValidationError('Obsolete definition')
        return super(MutableModel, self).clean()

    def save(self, *args, **kwargs):
        if self.is_obsolete():
            msg = _('Cannot save an obsolete model')
            raise ValidationError(msg)
        return super(MutableModel, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.is_obsolete():
            msg = _('Cannot delete an obsolete model')
            raise ValidationError(msg)
        return super(MutableModel, self).delete(*args, **kwargs)



from __future__ import unicode_literals

from django.core.management.commands.dumpdata import Command as BaseCommand

from ...models import ModelDefinition


class Command(BaseCommand):
    """
    `dumpdata` command override that makes sure to load all required mutable
    models in the cache prior to dumping.
    """

    def handle(self, *app_labels, **options):
        model_defs = ModelDefinition.objects.all()

        # Filter out non needed model definitions when some are specified.
        if app_labels:
            model_defs = model_defs.filter(
                app_label__in=set(
                    app_label.split('.')[0] for app_label in app_labels
                )
            )

        # Generate model class associated with model classes.
        for model_def in model_defs:
            model_def.model_class()

        return super(Command, self).handle(*app_labels, **options)



from __future__ import unicode_literals

from .postgresql_psycopg2 import *


DATABASES['default']['ENGINE'] = 'django.contrib.gis.db.backends.postgis'

INSTALLED_APPS.extend([
    'django.contrib.gis',
    'mutant.contrib.geo',
])

TEST_DEFAULT_LABELS.append('geo')

COVERAGE_MODULE_EXCLUDES.remove('mutant.contrib.geo')


from __future__ import unicode_literals

import os
import sys

from django.utils.translation import ugettext_lazy as _

from mutant.test import testcases
from mutant.tests.utils import BaseModelDefinitionTestCase

from . import models


PACKAGE_PATH = os.path.dirname(sys.modules[__name__].__file__)
MODULE_PATH = os.path.abspath(sys.modules[__name__].__file__)
MODELS_MODULE_PATH = os.path.abspath(models.__file__)


class FilePathFieldDefinitionTest(testcases.FieldDefinitionTestMixin,
                                  BaseModelDefinitionTestCase):
    field_definition_category = _('File')
    field_definition_cls = models.FilePathFieldDefinition
    field_definition_init_kwargs = {'path': PACKAGE_PATH}
    field_values = (MODULE_PATH, MODELS_MODULE_PATH)

    def test_formfield(self):
        self.field.match = r'\.pyc?$'
        self.field.save()
        formfield = self.field.field_instance().formfield()
        self.assertTrue(formfield.valid_value(MODULE_PATH))
        invalid_path = os.path.abspath(testcases.__file__)
        self.assertFalse(formfield.valid_value(invalid_path))



from __future__ import unicode_literals

import json
from StringIO import StringIO
from tempfile import NamedTemporaryFile

from django.core.management import call_command
from django.core.serializers.json import Serializer as JSONSerializer

from mutant.models import ModelDefinition
from mutant.test.testcases import ModelDefinitionDDLTestCase
from mutant.utils import remove_from_app_cache
from django.core.serializers.base import DeserializationError


class DataCommandTestCase(ModelDefinitionDDLTestCase):
    def setUp(self):
        self.model_def = ModelDefinition.objects.create(
            app_label='mutant', object_name='Model'
        )
        self.model_cls = self.model_def.model_class()

    def tearDown(self):
        if self.model_def.pk:
            self.model_def.delete()


class DumpDataTestCase(DataCommandTestCase):
    def dump_model_data(self):
        # Make sure to remove the model from the app cache because we're
        # actually testing it's correctly loaded.
        output = StringIO()
        remove_from_app_cache(self.model_cls)
        call_command(
            'dumpdata', str(self.model_def), stdout=output, commit=False
        )
        output.seek(0)
        return json.load(output)

    def test_dump_mutable_models(self):
        """
        Make sure mutable models instances are dumped when calling `dumpdata`.
        """
        self.assertEqual(self.dump_model_data(), [])
        instance = self.model_cls.objects.create()
        self.assertEqual(
            self.dump_model_data(), [{
                'pk': instance.pk,
                'model': str(self.model_def).lower(),
                'fields': {}
            }]
        )


class LoadDataTestCase(DataCommandTestCase):
    def setUp(self):
        super(LoadDataTestCase, self).setUp()
        self.serializer = JSONSerializer()

    def test_load_mutable_models(self):
        """
        Makes sure mutable models instances are correctly loaded when calling
        `loaddata`.
        """
        instance = self.model_cls(pk=1)
        # Make sure to remove the model from the app cache because we're
        # actually testing it's correctly loaded.
        remove_from_app_cache(self.model_cls)
        with NamedTemporaryFile(suffix='.json') as stream:
            self.serializer.serialize([instance], stream=stream)
            stream.seek(0)
            call_command(
                'loaddata', stream.name, stdout=StringIO(), commit=False
            )
        self.assertTrue(self.model_cls.objects.filter(pk=instance.pk).exists())

    def test_invalid_model_idenfitier_raises(self):
        """
        Makes sure an invalid model identifier raises the correct exception.
        """
        instance = self.model_cls(pk=1)
        with NamedTemporaryFile(suffix='.json') as stream:
            self.serializer.serialize([instance], stream=stream)
            stream.seek(0)
            self.model_def.delete()
            with self.assertRaisesMessage(
                    DeserializationError, "Invalid model identifier: 'mutant.model'"):
                call_command(
                    'loaddata', stream.name, stdout=StringIO()
                )



from __future__ import unicode_literals

import sys
import warnings
# TODO: Remove when support for Python 2.6 is dropped
if sys.version_info >= (2, 7):
    from unittest import TestCase
else:
    from django.utils.unittest import TestCase

from django.core.exceptions import ValidationError

from mutant.contrib.numeric.models import IntegerFieldDefinition
from mutant.contrib.text.models import CharFieldDefinition
from mutant.models.field import (FieldDefinition, FieldDefinitionChoice,
    NOT_PROVIDED)
from mutant.tests.utils import BaseModelDefinitionTestCase


class FieldDefinitionInheritanceTest(BaseModelDefinitionTestCase):
    def test_proxy_inheritance(self):
        obj = CharFieldDefinition.objects.create(name='caca',
                                                  max_length=25,
                                                  model_def=self.model_def)
        save_obj = self.model_def.fielddefinitions.select_subclasses().get()
        self.assertEqual(obj, save_obj)
        Model = self.model_def.model_class()
        Model.objects.create(caca="NO WAY")


class FieldDefinitionDeclarationTest(TestCase):
    def test_delete_override(self):
        """
        Make sure a warning is raised when declaring a `FieldDefinition`
        subclass that override the `delete` method.
        """
        with self.assertRaises(TypeError):
            with warnings.catch_warnings(record=True) as catched_warnings:
                class CustomFieldDefinition(FieldDefinition):
                    def delete(self, *args, **kwargs):
                        pass

                class CustomFieldDefinitionProxy(CustomFieldDefinition):
                    class Meta:
                        proxy = True

                    def delete(self, *args, **kwargs):
                        pass

        self.assertIn('Avoid overriding the `delete` method on '
                      '`FieldDefinition` subclass `CustomFieldDefinition`',
                      catched_warnings[0].message.args[0])


def module_level_pickable_default():
    module_level_pickable_default.incr += 1
    return module_level_pickable_default.incr
module_level_pickable_default.incr = 0


class FieldDefaultTest(BaseModelDefinitionTestCase):
    def test_clean(self):
        field = IntegerFieldDefinition(name='field', model_def=self.model_def)
        # Field cleaning should work when a default value isn't provided
        field.clean()
        with self.assertRaises(ValidationError):
            field.default = 'invalid'
            field.clean()
        field.default = module_level_pickable_default
        field.clean()
        field.save()
        Model = self.model_def.model_class()
        self.assertEqual(Model.objects.create().field,
                         module_level_pickable_default.incr)
        field.default = NOT_PROVIDED
        field.save()
        with self.assertRaises(ValidationError):
            obj = Model()
            obj.field
            obj.full_clean()

    def test_create_with_default(self):
        Model = self.model_def.model_class()
        Model.objects.create()
        IntegerFieldDefinition.objects.create_with_default(1337, name='field',
                                                           model_def=self.model_def)
        before = Model.objects.get()
        self.assertEqual(before.field, 1337)
        self.assertFalse(Model().field)


class FieldDefinitionChoiceTest(BaseModelDefinitionTestCase):
    def test_simple_choices(self):
        field_def = CharFieldDefinition.objects.create(name='gender',
                                                       max_length=1,
                                                       model_def=self.model_def)
        male_choice = FieldDefinitionChoice(field_def=field_def,
                                            value='Male', label='Male')
        # Value is longer than the max_length
        self.assertRaises(ValidationError, male_choice.clean)
        # A length of 1 should work
        male_choice.value = 'M'
        male_choice.full_clean()
        male_choice.save()
        # Cleaning should raise validation error when passed invalid choice
        Model = self.model_def.model_class()
        obj = Model(gender='T')
        self.assertRaises(ValidationError, obj.full_clean)
        # Create another allowed choice
        female_choice = FieldDefinitionChoice(field_def=field_def,
                                              value='F', label='Female')
        female_choice.value = 'F'
        female_choice.full_clean()
        female_choice.save()
        # It should now be possible to create valid objects with this choice
        obj = Model(gender='F')
        obj.full_clean()
        # Make sure choices are correctly set
        choices = Model._meta.get_field('gender').get_choices(include_blank=False)
        self.assertEqual(choices, [('M', 'Male'), ('F', 'Female')])

    def test_grouped_choices(self):
        field_def = CharFieldDefinition.objects.create(name='media',
                                                       max_length=5,
                                                       model_def=self.model_def)
        # Create Audio choices
        FieldDefinitionChoice.objects.create(field_def=field_def, group='Audio',
                                             value='vinyl', label='Vinyl')
        FieldDefinitionChoice.objects.create(field_def=field_def, group='Audio',
                                             value='cd', label='CD')
        # Create Video choices
        FieldDefinitionChoice.objects.create(field_def=field_def, group='Video',
                                             value='vhs', label='VHS Tape')
        FieldDefinitionChoice.objects.create(field_def=field_def, group='Video',
                                             value='dvd', label='DVD')
        # Create Unknown choices
        FieldDefinitionChoice.objects.create(field_def=field_def,
                                             value='unknown', label='Unknown')
        # Make sure choices are correctly created
        Model = self.model_def.model_class()
        choices = Model._meta.get_field('media').get_choices(include_blank=False)
        expected_choices = [
            ('Audio', (
                    ('vinyl', 'Vinyl'),
                    ('cd', 'CD'),
                )
             ),
            ('Video', (
                    ('vhs', 'VHS Tape'),
                    ('dvd', 'DVD'),
                )
             ),
            ('unknown', 'Unknown')
        ]
        self.assertEqual(choices, expected_choices)


class FieldDefinitionManagerTest(BaseModelDefinitionTestCase):
    def test_natural_key(self):
        fd = CharFieldDefinition.objects.create(name='name', max_length=5,
                                                model_def=self.model_def)
        natural_key = fd.natural_key()
        self.assertEqual(
            FieldDefinition.objects.get_by_natural_key(*natural_key), fd
        )



from __future__ import unicode_literals

from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models.field import FieldDefinition


class _BooleanMeta:
    defined_field_category = _('Boolean')


class BooleanFieldDefinition(FieldDefinition):
    class Meta(_BooleanMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.BooleanField


class NullBooleanFieldDefinition(FieldDefinition):
    class Meta(_BooleanMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.NullBooleanField



from __future__ import unicode_literals


def patch_model_option_verbose_name_raw():
    """
    Until #17763 and all the permission name length issues are fixed we patch
    the `verbose_name_raw` method to return a truncated string in order to
    avoid DatabaseError.
    """
    from django.db.models.options import Options
    verbose_name_raw = Options.verbose_name_raw.fget
    if hasattr(verbose_name_raw, '_patched'):
        return

    def _get_verbose_name_raw(self):
        name = verbose_name_raw(self)
        if len(name) >= 40:
            name = "%s..." % name[0:36]
        return name
    _get_verbose_name_raw.patched = True
    Options.verbose_name_raw = property(_get_verbose_name_raw)



from __future__ import unicode_literals

import logging
from optparse import make_option

from django.conf import settings
from django.test.simple import DjangoTestSuiteRunner

from mutant import logger


class MutantTestSuiteRunner(DjangoTestSuiteRunner):
    option_list = (
        make_option('-l', '--logger-level',
            dest='logger_level',
            help='Set the level of the `mutant` logger.'),
    )

    def __init__(self, logger_level, **kwargs):
        super(MutantTestSuiteRunner, self).__init__(**kwargs)
        if logger_level:
            logger.setLevel(logger_level)
            logger.addHandler(logging.StreamHandler())

    def build_suite(self, test_labels, extra_tests=None, **kwargs):
        if not test_labels:
            test_labels = getattr(settings, 'TEST_DEFAULT_LABELS', test_labels)
        return super(MutantTestSuiteRunner, self).build_suite(
            test_labels, extra_tests=None, **kwargs
        )



from __future__ import unicode_literals

import sys
# TODO: Remove when support for Python 2.6 is dropped
if sys.version_info >= (2, 7):
    from unittest import TestCase
else:
    from django.utils.unittest import TestCase

from django.core.exceptions import ImproperlyConfigured
from django.db import models
from django.utils.translation import ugettext_lazy as _

from mutant.db.fields.related import ModelClassAttributeDescriptor
from mutant.db.fields.translation import LazilyTranslatedField
from mutant.models import ModelDefinition

from .utils import BaseModelDefinitionTestCase


class LazilyTranslatedFieldTest(TestCase):
    @classmethod
    def setUpClass(cls):
        cls.field = LazilyTranslatedField()
        return super(LazilyTranslatedFieldTest, cls).setUpClass()

    def test_to_python(self):
        self.assertIsNone(self.field.to_python(None))
        self.assertEqual(self.field.to_python(_('hello')), _('hello'))
        self.assertEqual(self.field.to_python('hello'), _('hello'))
        self.assertEqual(self.field.to_python('hello'), _('hello'))
        self.assertEqual(self.field.to_python(1), _('1'))

    def test_get_prep_value(self):
        self.assertIsNone(self.field.get_prep_value(None))
        self.assertEqual(self.field.get_prep_value(_('hello')), 'hello')
        self.assertEqual(self.field.get_prep_value('hello'), 'hello')
        self.assertEqual(self.field.get_prep_value('hello'), 'hello')
        self.assertEqual(self.field.get_prep_value(1), '1')


class ModelWithModelDefinitionReference(models.Model):
    model_def = models.OneToOneField(ModelDefinition, related_name='+')
    model_objects = ModelClassAttributeDescriptor('model_def', 'objects')

    nullable_model_def = models.ForeignKey(
        ModelDefinition, related_name='+', null=True
    )
    nullable_objects = ModelClassAttributeDescriptor(
        'nullable_model_def', 'objects'
    )

    class Meta:
        app_label = 'mutant'


class ModelDefinitionReferenceTest(BaseModelDefinitionTestCase):
    def test_manager_name_clash(self):
        # Inexistent field
        with self.assertRaises(ImproperlyConfigured):
            class InexistentModelDefField(models.Model):
                objs = ModelClassAttributeDescriptor('model_def', 'objects')
        # Non-FK field
        with self.assertRaises(ImproperlyConfigured):
            class NonFKModelDefField(models.Model):
                name = models.CharField(max_length=100)
                objs = ModelClassAttributeDescriptor('name', 'objects')
        # FK not pointing to ModelDefinition
        with self.assertRaises(ImproperlyConfigured):
            class NonModelDefFKField(models.Model):
                model_def = models.ForeignKey('self')
                objs = ModelClassAttributeDescriptor('model_def', 'objects')

    def test_manager_descriptor(self):
        obj = ModelWithModelDefinitionReference()
        # Not nullable field definition should raise
        with self.assertRaises(AttributeError):
            obj.model_objects
        # Nullable field definition should raise
        with self.assertRaises(AttributeError):
            obj.nullable_objects
        # Assigning an existing model def should allow manager retrieval
        obj.model_def = self.model_def
        self.assertIsInstance(obj.model_objects, models.Manager)
        # Assigning an existing model def should allow manager retrieval
        obj.nullable_model_def = self.model_def
        self.assertIsInstance(obj.nullable_objects, models.Manager)
        # Making sure we've got the right model
        Model = self.model_def.model_class()
        Model.objects.create()
        self.assertEqual(obj.model_objects.count(), 1)
        self.assertEqual(obj.nullable_objects.count(), 1)



from __future__ import unicode_literals

from django.db import models


class ModelDefinitionManager(models.Manager):
    use_for_related_fields = True

    def get_by_natural_key(self, app_label, model):
        return self.get(app_label=app_label, model=model)



from __future__ import unicode_literals

from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models import FieldDefinition, FieldDefinitionManager


auto_now_help_text = _('Automatically set the field to now every time the '
                       'object is saved.')
auto_now_add_help_text = _('Automatically set the field to now when the '
                           'object is first created.')

class DateFieldDefinition(FieldDefinition):
    auto_now = fields.BooleanField(_('auto now'), default=False,
                                   help_text=auto_now_help_text)
    auto_now_add = fields.BooleanField(_('auto now add'), default=False,
                                       help_text=auto_now_add_help_text)

    objects = FieldDefinitionManager()

    class Meta:
        app_label = 'mutant'
        defined_field_class = fields.DateField
        defined_field_options = ('auto_now', 'auto_now_add',)
        defined_field_category = _('Temporal')


class TimeFieldDefinition(DateFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.TimeField


class DateTimeFieldDefinition(DateFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.DateTimeField



from __future__ import unicode_literals

from contextlib import contextmanager
from copy import deepcopy
from itertools import groupby
import imp
from operator import itemgetter

import django
from django.db.models.loading import cache as app_cache
from django.utils.datastructures import SortedDict
from django.utils.encoding import force_unicode
from django.utils.functional import lazy


NOT_PROVIDED = object()


def popattr(obj, attr, default=NOT_PROVIDED):
    """
    Useful for retrieving an object attr and removing it if it's part of it's
    dict while allowing retrieving from subclass.
    i.e.
    class A:
        a = 'a'
    class B(A):
        b = 'b'
    >>> popattr(B, 'a', None)
    'a'
    >>> A.a
    'a'
    """
    val = getattr(obj, attr, default)
    try:
        delattr(obj, attr)
    except AttributeError:
        if default is NOT_PROVIDED:
            raise
    return val


def _string_format(string, *args, **kwargs):
    if args:
        return string % tuple(force_unicode(s) for s in args)
    elif kwargs:
        return string % dict((k, force_unicode(v)) for k, v in kwargs.iteritems())
lazy_string_format = lazy(_string_format, unicode)


def get_db_table(app_label, model):
    return "mutant_%s_%s" % (app_label, model)


@contextmanager
def app_cache_lock():
    try:
        imp.acquire_lock()
        yield
    finally:
        imp.release_lock()


def remove_from_app_cache(model_class):
    opts = model_class._meta
    app_label, model_name = opts.app_label, opts.object_name.lower()
    with app_cache_lock():
        app_models = app_cache.app_models.get(app_label, False)
        if app_models:
            model = app_models.pop(model_name, False)
            if model:
                app_cache._get_models_cache.clear()
                return model


def _app_cache_deepcopy(obj):
    """
    An helper that correctly deepcopy model cache state
    """
    if isinstance(obj, dict):
        return dict((_app_cache_deepcopy(key), _app_cache_deepcopy(val))
                    for key, val in obj.iteritems())
    elif isinstance(obj, list):
        return list(_app_cache_deepcopy(val) for val in obj)
    elif isinstance(obj, SortedDict):
        return deepcopy(obj)
    return obj


@contextmanager
def app_cache_restorer():
    """
    A context manager that restore model cache state as it was before
    entering context.
    """
    state = _app_cache_deepcopy(app_cache.__dict__)
    try:
        yield state
    finally:
        with app_cache_lock():
            app_cache.__dict__ = state


group_item_getter = itemgetter('group')


def choices_from_dict(choices):
    for grp, choices in groupby(choices, key=group_item_getter):
        if grp is None:
            for choice in choices:
                yield (choice['value'], choice['label'])
        else:
            yield (grp, tuple((choice['value'], choice['label'])
                                for choice in choices))


# TODO: Remove when support for 1.5 is dropped
if django.VERSION >= (1, 6):  # pragma: no cover
    def model_name(opts):
        return opts.model_name
else:  # pragma: no cover
    def model_name(opts):
        return opts.module_name



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import (EmailFieldDefinition, GenericIPAddressFieldDefinition,
    IPAddressFieldDefinition, SlugFieldDefinition, URLFieldDefinition)


class WebFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Web')


class EmailFieldDefinitionTest(WebFieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = EmailFieldDefinition
    field_values = ('guido@python.org', 'god@heaven.com')


class IPAddressFieldDefinitionTest(WebFieldDefinitionTestMixin,
                                   BaseModelDefinitionTestCase):
    field_definition_cls = IPAddressFieldDefinition
    field_definition_init_kwargs = {'default': '192.168.1.1'}
    field_values = ('127.0.0.1', '82.94.164.162')


class SlugFieldDefinitionTest(WebFieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = SlugFieldDefinition
    field_values = (
        'an-awesome-slug_-_-',
        '2012-4-7-so-late'
    )


class URLFieldDefinitionTest(WebFieldDefinitionTestMixin,
                             BaseModelDefinitionTestCase):
    field_definition_cls = URLFieldDefinition
    field_values = (
        'https://github.com/charettes/django-mutant',
        'http://travis-ci.org/#!/charettes/django-mutant',
    )


class GenericIPAddressFieldDefinitionTest(WebFieldDefinitionTestMixin,
                                          BaseModelDefinitionTestCase):
    field_definition_cls = GenericIPAddressFieldDefinition
    field_values = (
        '127.0.0.1',
        '2001:db8:85a3::8a2e:370:7334'
    )



from __future__ import unicode_literals

from django.contrib.gis.db import models
from django.utils.translation import ugettext_lazy as _

from mutant.models import FieldDefinition, FieldDefinitionManager


srid_help_text = _('Spatial Reference System Identity')
spatial_index_help_text = _('Creates a spatial index for the given '
                            'geometry field.')
dim_help_text = _('Coordinate dimension.')
geography_help_text = _('Creates a database column of type geography, '
                        'rather than geometry.')


class GeometryFieldDefinition(FieldDefinition):
    DIM_2D = 2
    DIM_3D = 3
    DIM_CHOICES = (
        (DIM_2D, _('Two-dimensional')),
        (DIM_3D, _('Three-dimensional')),
    )

    srid = models.IntegerField(_('SRID'), default=4326,
                               help_text=srid_help_text)
    spatial_index = models.BooleanField(_('spatial index'), default=True,
                                        help_text=spatial_index_help_text)
    dim = models.PositiveSmallIntegerField(_('coordinate dimension'),
                                           choices=DIM_CHOICES, default=2,
                                           help_text=dim_help_text)
    geography = models.BooleanField(_('geography'), default=False,
                                    help_text=geography_help_text)

    objects = FieldDefinitionManager()

    class Meta:
        app_label = 'mutant'
        defined_field_options = ('srid', 'spatial_index', 'dim', 'geography')
        defined_field_category = _('Geometry')
        defined_field_class = models.GeometryField


class PointFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.PointField


class LineStringFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.LineStringField


class PolygonFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.PolygonField


class MultiPointFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.MultiPointField


class MultiLineStringFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.MultiLineStringField


class MultiPolygonFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.MultiPolygonField


class GeometryCollectionFieldDefinition(GeometryFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = models.GeometryCollectionField



from __future__ import unicode_literals

from inspect import isclass

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.validators import EMPTY_VALUES
from django.forms.fields import ChoiceField
from django.utils.encoding import force_text, smart_text

from .models.field import FieldDefinition, FieldDefinitionBase
from .utils import group_item_getter, choices_from_dict


class FieldDefinitionTypeField(ChoiceField):
    def __init__(self, field_definitions=None, empty_label="---------",
                 group_by_category=True, *args, **kwargs):
        if field_definitions is None:
            field_definitions = FieldDefinitionBase._field_definitions.values()
        else:
            for fd in field_definitions:
                if not isinstance(fd, FieldDefinitionBase):
                    raise TypeError("%r is not a subclass of FieldDefinitionBase" % fd)
        fds_choices = []
        content_types = ContentType.objects.get_for_models(
            *field_definitions, for_concrete_models=False
        )
        for fd, ct in content_types.iteritems():
            group = force_text(fd.get_field_category()) if group_by_category else None
            fds_choices.append({
                'value': ct.pk,
                'label': force_text(fd.get_field_description()),
                'group': group,
            })
        choices = [('', empty_label)] + list(choices_from_dict(sorted(fds_choices, key=group_item_getter)))
        super(FieldDefinitionTypeField, self).__init__(choices, *args, **kwargs)

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return None
        try:
            ct = ContentType.objects.get_for_id(value)
        except ContentType.DoesNotExist:
            raise ValidationError(self.error_messages['invalid_choice'])
        return ct

    def valid_value(self, value):
        if isclass(value) and issubclass(value, FieldDefinition):
            value = value.get_content_type()
        if isinstance(value, ContentType):
            value = value.pk
        value = smart_text(value)
        return super(FieldDefinitionTypeField, self).valid_value(value)



from south.modelsinspector import add_introspection_rules


add_introspection_rules([], ['^mutant\.db\.fields\.python\.PythonIdentifierField'])
add_introspection_rules([], ['^mutant\.db\.fields\.python\.RegExpStringField'])
add_introspection_rules([], ['^mutant\.db\.fields\.python\.DirectoryPathField'])
add_introspection_rules([], ['^mutant\.db\.fields\.translation\.LazilyTranslatedField'])



from __future__ import unicode_literals

import re

from django.core.validators import RegexValidator
from django.utils.translation import ugettext_lazy as _


python_identifier_re = re.compile(r'^[a-z_][\w_]*$', re.IGNORECASE)
validate_python_identifier = RegexValidator(python_identifier_re,
                                            _('Enter a valid python identifier.'),
                                            'invalid')

python_object_path_re = re.compile(r'^[a-z_]+(\.[\w_]+)*$', re.IGNORECASE)
validate_python_object_path = RegexValidator(python_object_path_re,
                                             _('Enter a valid python object path.'),
                                             'invalid')



from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db.models import deletion, fields
from django.utils.translation import ugettext_lazy as _
from picklefield.fields import PickledObjectField

from ...db.fields import PythonIdentifierField
from ...db.models import MutableModel
from ...models import FieldDefinition, FieldDefinitionManager, ModelDefinition

from .managers import ForeignKeyDefinitionManager


related_name_help_text = _('The name to use for the relation from the '
                           'related object back to this one.')

class RelatedFieldDefinition(FieldDefinition):
    to = fields.related.ForeignKey(ContentType, verbose_name=_('to'),
                                   related_name='+')
    related_name = PythonIdentifierField(_('related name'),
                                         blank=True, null=True,
                                         help_text=related_name_help_text)

    objects = FieldDefinitionManager()

    class Meta:
        app_label = 'mutant'
        abstract = True
        defined_field_options = ('related_name',)
        defined_field_category = _('Related')

    def clone(self):
        clone = super(RelatedFieldDefinition, self).clone()
        clone.to = self.to
        return clone

    @property
    def is_recursive_relationship(self):
        """
        Whether or not `to` points to this field's model definition
        """
        try:
            model_def = self.model_def
        except ModelDefinition.DoesNotExist:
            return False
        else:
            return self.to_id == model_def.contenttype_ptr_id

    @property
    def to_model_class_is_mutable(self):
        to_model_class = self.to.model_class()
        if to_model_class is None:
            try:
                getattr(self.to, 'modeldefinition')
            except ModelDefinition.DoesNotExist:
                return False
            else:
                return True
        else:
            return issubclass(to_model_class, MutableModel)

    @property
    def to_model_class(self):
        if self.to_model_class_is_mutable:
            return self.to.modeldefinition.model_class()
        else:
            return self.to.model_class()

    def clean(self):
        if (None not in (self.related_name, self.to_id) and
            not self.to_model_class_is_mutable):
            msg = _('Cannot assign a related manager to non-mutable model')
            raise ValidationError({'related_name': [msg]})

    def get_field_options(self, **overrides):
        options = super(RelatedFieldDefinition, self).get_field_options(**overrides)
        if self.is_recursive_relationship:
            options['to'] = fields.related.RECURSIVE_RELATIONSHIP_CONSTANT
        else:
            opts = self.to_model_class._meta
            options['to'] = "%s.%s" % (opts.app_label, opts.object_name)
        if not self.to_model_class_is_mutable:
            options['related_name'] = '+'
        return options

    def _south_ready_field_instance(self):
        """
        South add_column choke when passing 'self' or 'app.Model' to `to` kwarg,
        so we have to create a special version for it.
        """
        cls = self.get_field_class()
        options = self.get_field_options()
        options['to'] = self.to.model_class()
        return cls(**options)


to_field_help_text = _('The field on the related object that the '
                       'relation is to.')
on_delete_help_text = _('Behavior when an object referenced by this field '
                        'is deleted')

class ForeignKeyDefinition(RelatedFieldDefinition):
    ON_DELETE_CASCADE = 'CASCADE'
    ON_DELETE_PROTECT = 'PROTECT'
    ON_DELETE_SET_NULL = 'SET_NULL'
    ON_DELETE_SET_DEFAULT = 'SET_DEFAULT'
    ON_DELETE_SET_VALUE = 'SET_VALUE'
    ON_DELETE_DO_NOTHING = 'DO_NOTHING'

    ON_DELETE_CHOICES = (
        (ON_DELETE_CASCADE, _('CASCADE')),
        (ON_DELETE_PROTECT, _('PROTECT')),
        (ON_DELETE_SET_NULL, _('SET_NULL')),
        (ON_DELETE_SET_DEFAULT, _('SET_DEFAULT')),
        (ON_DELETE_SET_VALUE, _('SET(VALUE)')),
        (ON_DELETE_DO_NOTHING, _('DO_NOTHING')),
    )

    to_field = PythonIdentifierField(_('to field'), blank=True, null=True,
                                     help_text=to_field_help_text)
    one_to_one = fields.BooleanField(editable=False, default=False)
    on_delete = fields.CharField(_('on delete'), blank=True, null=True,
                                 max_length=11, choices=ON_DELETE_CHOICES,
                                 default=ON_DELETE_CASCADE,
                                 help_text=on_delete_help_text)
    on_delete_set_value = PickledObjectField(_('on delete set value'), null=True)

    objects = ForeignKeyDefinitionManager(one_to_one=False)

    class Meta:
        app_label = 'mutant'
        defined_field_class = fields.related.ForeignKey
        defined_field_options = ('to_field',)

    def clean(self):
        try:
            super(ForeignKeyDefinition, self).clean()
        except ValidationError as e:
            messages = e.message_dict
        else:
            messages = {}
        if self.on_delete == self.ON_DELETE_SET_NULL:
            if not self.null:
                msg = _("This field can't be null")
                messages['on_delete'] = [msg]
        elif (self.on_delete == self.ON_DELETE_SET_DEFAULT and
              self.default == fields.NOT_PROVIDED):
            msg = _('This field has no default value')
            messages['on_delete'] = [msg]
        elif (self.on_delete == self.ON_DELETE_SET_VALUE and
              self.on_delete_set_value is None):
            msg = _('You must specify a value to set on deletion')
            messages['on_delete'] = [msg]
        if messages:
            raise ValidationError(messages)

    def get_field_options(self, **overrides):
        options = super(ForeignKeyDefinition, self).get_field_options(**overrides)
        if self.on_delete == self.ON_DELETE_SET_VALUE:
            on_delete = deletion.SET(self.on_delete_set_value)
        else:
            on_delete = getattr(deletion, self.on_delete, None)
        options['on_delete'] = on_delete
        return options


class OneToOneFieldDefinition(ForeignKeyDefinition):
    objects = ForeignKeyDefinitionManager(one_to_one=True)

    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.related.OneToOneField

    def save(self, *args, **kwargs):
        self.one_to_one = True
        return super(OneToOneFieldDefinition, self).save(*args, **kwargs)


through_help_text = _('Intermediary model')

db_table_help_text = _('The name of the table to create for storing the '
                       'many-to-many data')

class ManyToManyFieldDefinition(RelatedFieldDefinition):
    symmetrical = fields.NullBooleanField(_('symmetrical'))
    through = fields.related.ForeignKey(ContentType, blank=True, null=True,
                                        related_name='+',
                                        help_text=through_help_text)
    # TODO: This should not be a SlugField
    db_table = fields.SlugField(max_length=30, blank=True, null=True,
                                help_text=db_table_help_text)

    class Meta:
        app_label = 'mutant'
        defined_field_class = fields.related.ManyToManyField
        defined_field_options = ('symmetrical', 'db_table')

    def clean(self):
        try:
            super(ManyToManyFieldDefinition, self).clean()
        except ValidationError as e:
            messages = e.message_dict
        else:
            messages = {}

        if (self.symmetrical is not None and 
            not self.is_recursive_relationship):
            msg = _("The relationship can only be symmetrical or not if it's "
                    "recursive, i. e. it points to 'self'")
            messages['symmetrical'] = [msg]

        if self.through:
            if self.db_table:
                msg = _('Cannot specify a db_table if an intermediate '
                        'model is used.')
                messages['db_table'] = [msg]

            if self.symmetrical:
                msg = _('Many-to-many fields with intermediate model cannot '
                        'be symmetrical.')
                messages.setdefault('symmetrical', []).append(msg)

            seen_from, seen_to = 0, 0
            to_model = self.to.model_class()  
            through_class = self.through.model_class()
            from_model = self.model_def.cached_model
            for field in through_class._meta.fields:
                rel_to = getattr(field.rel, 'to', None)
                if rel_to == from_model:
                    seen_from += 1
                elif rel_to == to_model:
                    seen_to += 1
            if self.is_recursive_relationship():
                if seen_from > 2:
                    msg = _('Intermediary model %s has more than two foreign '
                            'keys to %s, which is ambiguous and is not permitted.')
                    formated_msg = msg % (through_class._meta.object_name,
                                          from_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)
            else:
                msg = _('Intermediary model %s has more than one foreign key '
                        ' to %s, which is ambiguous and is not permitted.')
                if seen_from > 1:
                    formated_msg = msg % (through_class._meta.object_name,
                                          from_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)
                if seen_to > 1:
                    formated_msg = msg % (through_class._meta.object_name,
                                          to_model._meta.object_name)
                    messages.setdefault('through', []).append(formated_msg)

        if messages:
            raise ValidationError(messages)

    def get_field_options(self, **overrides):
        options = super(ManyToManyFieldDefinition, self).get_field_options(**overrides)
        if self.through:
            options['through'] = self.through.model_class()
        return options

    def get_bound_field(self):
        opts = self.model_def.model_class(force_create=True)._meta
        for field in opts.many_to_many:
            if getattr(field, self.FIELD_DEFINITION_PK_ATTR, None) == self.pk:
                return field



from __future__ import unicode_literals

import datetime

from django.utils.translation import ugettext_lazy as _

from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import (DateFieldDefinition, DateTimeFieldDefinition,
    TimeFieldDefinition)


class TemporalFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Temporal')


class DateFieldDefinitionTest(TemporalFieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = DateFieldDefinition
    field_definition_init_kwargs = {'default': datetime.date(1990, 8, 31)}
    field_values = (
        datetime.date.today(),
        datetime.date(1988, 5, 15)
    )


class DateTimeFieldDefinitionTest(TemporalFieldDefinitionTestMixin,
                                  BaseModelDefinitionTestCase):
    field_definition_cls = DateTimeFieldDefinition
    field_definition_init_kwargs = {
        'default': datetime.datetime(1990, 8, 31, 23, 46)
    }
    field_values = (
        datetime.datetime(2020, 11, 15, 15, 34),
        datetime.datetime(1988, 5, 15, 15, 30)
    )


class TimeFieldDefinitionTest(TemporalFieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = TimeFieldDefinition
    field_definition_init_kwargs = {'default': datetime.time(1, 1)}
    field_values = (
        datetime.time(1, 58, 37),
        datetime.time(17, 56)
    )



from __future__ import unicode_literals

from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models.field import FieldDefinition
from ...models.field.managers import FieldDefinitionManager


class CharFieldDefinition(FieldDefinition):
    max_length = fields.PositiveSmallIntegerField(
        _('max length'), blank=True, null=True
    )

    objects = FieldDefinitionManager()

    class Meta:
        app_label = 'mutant'
        defined_field_class = fields.CharField
        defined_field_options = ('max_length',)
        defined_field_description = _('String')
        defined_field_category = _('Text')


class TextFieldDefinition(CharFieldDefinition):
    class Meta:
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.TextField



from __future__ import unicode_literals

from django.contrib.gis.db import models


class GeoModel(models.Model):
    """
    A model to be used as a BaseDefinition on ModelDefinition instance with
    GeometryFieldDefinition instances.
    """
    objects = models.GeoManager()

    class Meta:
        abstract = True



from __future__ import unicode_literals
import warnings

import django
from django.db import models


class FilteredQuerysetManager(models.Manager):
    def __init__(self, *args, **kwargs):
        self.args = args
        self.kwargs = kwargs
        super(FilteredQuerysetManager, self).__init__()

    def get_queryset(self):
        qs = super(FilteredQuerysetManager, self).get_queryset()
        return qs.filter(*self.args, **self.kwargs)

    if django.VERSION < (1, 8):
        if django.VERSION >= (1, 6):
            def get_query_set(self):
                warnings.warn(
                    "`FilteredQuerysetManager.get_query_set` is deprecated, "
                    "use `get_queryset` instead",
                    DeprecationWarning if django.VERSION >= (1, 7)
                        else PendingDeprecationWarning,
                    stacklevel=2
                )
                return FilteredQuerysetManager.get_queryset(self)
        else:
            def get_query_set(self):
                qs = super(FilteredQuerysetManager, self).get_query_set()
                return qs.filter(*self.args, **self.kwargs)



from __future__ import unicode_literals

import pickle

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.core.management import call_command
from django.db import connections, models, router
from django.db.utils import IntegrityError
from django.utils.translation import ugettext_lazy as _

from mutant.contrib.text.models import CharFieldDefinition
from mutant.contrib.related.models import ForeignKeyDefinition
from mutant.db.models import MutableModel
from mutant.models.model import (ModelDefinition, OrderingFieldDefinition,
    UniqueTogetherDefinition, BaseDefinition)
from mutant.test.utils import CaptureQueriesContext

from .utils import (BaseModelDefinitionTestCase,
    skipUnlessMutantModelDBFeature)


try:
    from test.test_support import captured_stderr
except ImportError:
    # python 2.6 doesn't provide this helper
    from contextlib import contextmanager
    import StringIO
    import sys

    @contextmanager
    def captured_stderr():
        stderr = sys.stderr
        try:
            sys.stderr = StringIO.StringIO()
            yield sys.stderr
        finally:
            sys.stderr = stderr


class Mixin(object):
    def method(self):
        return 'Mixin'


class ConcreteModel(models.Model):
    concrete_model_field = models.NullBooleanField()

    class Meta:
        app_label = 'mutant'


class ProxyModel(ConcreteModel):
    class Meta:
        proxy = True


class AbstractModel(models.Model):
    abstract_model_field = models.CharField(max_length=5)

    class Meta:
        abstract = True

    def method(self):
        return 'AbstractModel'


class AbstractConcreteModelSubclass(ConcreteModel):
    abstract_concrete_model_subclass_field = models.CharField(max_length=5)

    class Meta:
        abstract = True


class ModelSubclassWithTextField(models.Model):
    abstract_model_field = models.TextField()
    second_field = models.NullBooleanField()

    class Meta:
        abstract = True


class ModelDefinitionTest(BaseModelDefinitionTestCase):
    def test_model_class_creation_cache(self):
        Model = self.model_def.model_class()
        self.assertEqual(Model, self.model_def.model_class())
        self.assertNotEqual(Model, self.model_def.model_class(force_create=True))

    def test_repr(self):
        """
        Make sure ModelDefinition objects are always repr()-able.
        """
        repr(self.model_def)
        repr(ModelDefinition())

    def get_model_db_table_name(self, model_def):
        model_class = model_def.model_class()
        return router.db_for_write(model_class), model_class._meta.db_table

    def test_rename_model(self):
        """
        Make sure changing the app_label or object_name renames the associated
        table
        """
        db, table_name = self.get_model_db_table_name(self.model_def)
        self.model_def.app_label = 'myapp'
        self.model_def.save()
        self.assertTableDoesntExists(db, table_name)
        db, table_name = self.get_model_db_table_name(self.model_def)
        self.assertTableExists(db, table_name)

        self.model_def.object_name = 'MyModel'
        self.model_def.save()
        self.assertTableDoesntExists(db, table_name)
        db, table_name = self.get_model_db_table_name(self.model_def)
        self.assertTableExists(db, table_name)

        self.model_def.delete()
        self.assertTableDoesntExists(db, table_name)

    def test_db_table(self):
        """
        Asserts that the db_table field is correctly handled
        """
        db, table_name = self.get_model_db_table_name(self.model_def)
        self.model_def.db_table = 'test_db_table'
        self.model_def.save()
        self.assertTableDoesntExists(db, table_name)
        self.assertTableExists(db, 'test_db_table')
        self.model_def.db_table = None
        self.model_def.save()
        self.assertTableDoesntExists(db, 'test_db_table')
        self.assertTableExists(db, table_name)

    def test_fixture_loading(self):
        """
        Make model and field definitions can be loaded from fixtures.
        """
        call_command(
            'loaddata', 'fixture_loading_test', verbosity=0, commit=False
        )
        self.assertTrue(
            ModelDefinition.objects.filter(
                app_label='myfixtureapp', object_name='MyFixtureModel'
            ).exists()
        )
        model_def = ModelDefinition.objects.get(
            app_label='myfixtureapp', object_name='MyFixtureModel'
        )
        MyFixtureModel = model_def.model_class()
        self.assertModelTablesExist(MyFixtureModel)
        # Makes sure concrete field definition subclasses are created...
        self.assertTrue(
            model_def.fielddefinitions.filter(
                name='fixture_charfield'
            ).exists()
        )
        # and their column is created.
        self.assertModelTablesColumnExists(MyFixtureModel, 'fixture_charfieldcolumn')
        # Makes sure proxy field definition subclasses are created...
        self.assertTrue(
            model_def.fielddefinitions.filter(
                name='fixture_integerfield'
            ).exists()
        )
        # and their column is created.
        self.assertModelTablesColumnExists(MyFixtureModel, 'fixture_integerfieldcolumn')

    def test_verbose_name(self):
        Model = self.model_def.model_class()

        self.assertEqual(Model._meta.verbose_name, self.model_def.model)
        self.assertEqual(Model._meta.verbose_name_plural, "%ss" % self.model_def.model)

        self.model_def.verbose_name = 'MyMoDeL'
        self.model_def.verbose_name_plural = 'MyMoDeLZ0Rs'
        self.model_def.save()

        self.assertEqual(Model._meta.verbose_name, _('MyMoDeL'))
        self.assertEqual(Model._meta.verbose_name_plural, _('MyMoDeLZ0Rs'))

    def test_multiple_model_definition(self):
        """
        Make sure multiple model definition can coexists
        """
        other_model_def = ModelDefinition.objects.create(app_label='app',
                                                         object_name='OtherModel')
        self.assertNotEqual(other_model_def.model_class(),
                            self.model_def.model_class())
        self.assertNotEqual(other_model_def.model_ct, self.model_def.model_ct)

    def test_natural_key(self):
        natural_key = self.model_def.natural_key()
        self.assertEqual(ModelDefinition.objects.get_by_natural_key(*natural_key),
                         self.model_def)

    def test_deletion(self):
        # Add a an extra field to make sure no alter statements are issued
        CharFieldDefinition.objects.create(
            model_def=self.model_def,
            name='field',
            max_length=10
        )
        # Add a base with a field to make sure no alter statements are issued
        BaseDefinition.objects.create(
            model_def=self.model_def,
            base=AbstractModel
        )
        model_cls = self.model_def.model_class()
        self.assertModelTablesExist(model_cls)
        db, table_name = self.get_model_db_table_name(self.model_def)
        connection = connections[db]
        with CaptureQueriesContext(connection) as captured_queries:
            self.model_def.delete()
        # Ensure no ALTER queries were issued during deletion of model_def,
        # that is, check that the columns were not deleted on table one at a
        # time before the entire table was dropped.
        self.assertFalse(any('ALTER' in query['sql'] for query in captured_queries))
        self.assertTableDoesntExists(db, table_name)

    def test_model_management(self):
        """
        Make sure no DDL is executed when a model is marked as managed.
        """
        model_def = self.model_def
        CharFieldDefinition.objects.create(
            model_def=model_def,
            name='field',
            max_length=10
        )
        model_cls = model_def.model_class()
        model_cls.objects.create(field='test')
        # Mark the existing model definition as `managed`.
        model_def.managed = True
        model_def.save()
        # Deleting a managed model shouldn't issue a DROP TABLE.
        db, table_name = self.get_model_db_table_name(self.model_def)
        model_def.delete()
        self.assertTableExists(db, table_name)
        # Attach a new model definition to the existing table
        new_model_def = ModelDefinition.objects.create(
            app_label=model_def.app_label,
            object_name=model_def.object_name,
            managed=True,
            fields=(CharFieldDefinition(name='field', max_length=10),)
        )
        # Make sure old data can be retrieved
        self.assertEqual(1, new_model_def.model_class().objects.count())
        # Mark the new model as unmanaged to make sure it's associated
        # table is deleted on tear down.
        new_model_def.managed = False
        new_model_def.save()


class ModelDefinitionManagerTest(BaseModelDefinitionTestCase):
    def test_fields_creation(self):
        char_field = CharFieldDefinition(name='name', max_length=10)
        ct_ct = ContentType.objects.get_for_model(ContentType)
        fk_field = ForeignKeyDefinition(name='ct', to=ct_ct)
        model_def = ModelDefinition.objects.create(
            app_label='app', object_name='OtherModel',
            fields=[char_field, fk_field]
        )
        model_cls = model_def.model_class()
        db = router.db_for_write(model_cls)
        table = model_cls._meta.db_table
        column = model_cls._meta.get_field('name').get_attname_column()[1]
        # Make sure column was created
        self.assertColumnExists(db, table, column)
        # Make sure field definitions were created
        self.assertIsNotNone(char_field.pk)
        self.assertIsNotNone(fk_field.pk)

    def test_bases_creation(self):
        mixin_base = BaseDefinition(base=Mixin)
        abstract_base = BaseDefinition(base=AbstractModel)
        abstract_concrete_base = BaseDefinition(
            base=AbstractConcreteModelSubclass
        )
        model_def = ModelDefinition.objects.create(
            app_label='app', object_name='OtherModel',
            bases=[mixin_base, abstract_base, abstract_concrete_base],
        )
        model = model_def.model_class()
        self.assertModelTablesColumnExists(
            model, 'abstract_model_field'
        )
        self.assertModelTablesColumnDoesntExists(
            model, 'concrete_model_field'
        )
        self.assertModelTablesColumnExists(
            model, 'abstract_concrete_model_subclass_field'
        )

    def test_primary_key_override(self):
        field = CharFieldDefinition(
            name='name', max_length=32, primary_key=True
        )
        model_def = ModelDefinition.objects.create(
            fields=[field], app_label='app', object_name='OtherModel'
        )
        self.assertEqual(model_def.model_class()._meta.pk.name, field.name)

    def test_get_or_create(self):
        """
        Make sure bases and fields defaults are reaching the model initializer.
        """
        field = CharFieldDefinition(name='name', max_length=32)
        base = BaseDefinition(base=AbstractModel)
        ModelDefinition.objects.get_or_create(
            app_label='app', object_name='OtherModel',
            defaults={'bases': [base], 'fields': [field]}
        )
        self.assertIsNotNone(field.pk)
        self.assertIsNotNone(base.pk)


class ModelValidationTest(BaseModelDefinitionTestCase):
    def test_installed_app_override_failure(self):
        """
        Make sure we can't save a model definition with an app_label of
        an installed app.
        """
        self.model_def.app_label = 'mutant'
        self.assertRaises(ValidationError, self.model_def.clean)


class ModelClassProxyTest(BaseModelDefinitionTestCase):
    def test_pickling(self):
        """
        Make sure _ModelClassProxy can be pickled correctly. This is required
        to allow a model definition to subclass a MutableModel.
        """
        Model = self.model_def.model_class()
        pickled = pickle.dumps(Model)
        self.assertEqual(pickle.loads(pickled), Model)

    def test_proxy_interactions(self):
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name="name", max_length=10)
        Model = self.model_def.model_class()
        sergei = Model.objects.create(name='Sergei')
        halak = Model(name='Halak')
        halak.save()
        self.assertTrue(issubclass(Model, models.Model))
        self.assertTrue(issubclass(Model, MutableModel))
        self.assertEqual("<class 'mutant.apps.app.models.Model'>",
                         unicode(Model))
        self.assertEqual(sergei, Model.objects.get(name='Sergei'))

        class A(object):
            class_model = Model

            def __init__(self, model):
                self.model = model

        a = A(Model)

        self.assertEqual(Model, a.model)
        self.assertEqual(Model, A.class_model)

        a.model = Model  # Assign a proxy
        a.model = a.model  # Assign a Model
        a.model = 4

    def test_definition_deletion(self):
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name="name", max_length=10)

        Model = self.model_def.model_class()
        db = router.db_for_write(Model)
        instance = Model.objects.create(name="Quebec")
        table_name = Model._meta.db_table
        self.model_def.delete()
        self.assertTableDoesntExists(db, table_name)

        with self.assertRaises(AttributeError):
            Model(name="name")

        with self.assertRaises(AttributeError):
            Model.objects.all()

        with self.assertRaises(ValidationError):
            instance.clean()

        with self.assertRaises(ValidationError):
            instance.save()

        with self.assertRaises(ValidationError):
            instance.delete()


class OrderingDefinitionTest(BaseModelDefinitionTestCase):
    def setUp(self):
        super(OrderingDefinitionTest, self).setUp()
        self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f1', max_length=25)
        ct_ct = ContentType.objects.get_for_model(ContentType)
        self.f2 = ForeignKeyDefinition.objects.create(model_def=self.model_def,
                                                      null=True,
                                                      name='f2', to=ct_ct)

    def test_clean(self):
        ordering = OrderingFieldDefinition(model_def=self.model_def)
        # Random
        ordering.lookup = '?'
        ordering.clean()
        # By f1
        ordering.lookup = 'f1'
        ordering.clean()
        # By f2 app label
        ordering.lookup = 'f2__app_label'
        ordering.clean()
        # Inexistent field
        with self.assertRaises(ValidationError):
            ordering.lookup = 'inexistent_field'
            ordering.clean()
        # Inexistent field of an existent field
        with self.assertRaises(ValidationError):
            ordering.lookup = 'f2__higgs_boson'
            ordering.clean()

    @skipUnlessMutantModelDBFeature('supports_joins')
    def test_simple_ordering(self):
        Model = self.model_def.model_class()
        model_ct = ContentType.objects.get_for_model(Model)  # app
        ct_ct = ContentType.objects.get_for_model(ContentType)  # contenttypes
        Model.objects.create(f1='Simon', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=model_ct)
        # Instances should be sorted by id
        self.assertSequenceEqual(
            Model.objects.values_list('f1', flat=True),
            ('Simon', 'Alexander')
        )
        # Instances should be sorted by f1 and not id
        f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f1')
        self.assertSequenceEqual(
            Model.objects.values_list('f1', flat=True),
            ('Alexander', 'Simon')
        )
        # Swap the ordering to descending
        f1_ordering.descending = True
        f1_ordering.save()
        self.assertSequenceEqual(
            Model.objects.values_list('f1', flat=True),
            ('Simon', 'Alexander')
        )
        f1_ordering.delete()
        # Order by f2__app_label
        f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f2__app_label')
        self.assertSequenceEqual(
            Model.objects.values_list('f1', flat=True),
            ('Alexander', 'Simon')
        )
        # Swap the ordering to descending
        f2_ordering.descending = True
        f2_ordering.save()
        self.assertSequenceEqual(
            Model.objects.values_list('f1', flat=True),
            ('Simon', 'Alexander')
        )
        f2_ordering.delete()

    @skipUnlessMutantModelDBFeature('supports_joins')
    def test_multiple_ordering(self):
        Model = self.model_def.model_class()
        model_ct = ContentType.objects.get_for_model(Model)  # app
        ct_ct = ContentType.objects.get_for_model(ContentType)  # contenttypes
        Model.objects.create(f1='Simon', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=model_ct)
        Model.objects.create(f1='Julia', f2=ct_ct)
        Model.objects.create(f1='Alexander', f2=ct_ct)
        # Orderings
        f1_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f1')
        f2_ordering = OrderingFieldDefinition.objects.create(model_def=self.model_def,
                                                             lookup='f2__app_label')
        self.assertSequenceEqual(
            Model.objects.values_list('f1', 'f2__app_label'), (
                ('Alexander', 'app'),
                ('Alexander', 'contenttypes'),
                ('Julia', 'contenttypes'),
                ('Simon', 'contenttypes')
            )
        )
        # Swap the ordering to descending
        f2_ordering.descending = True
        f2_ordering.save()
        self.assertSequenceEqual(
            Model.objects.values_list('f1', 'f2__app_label'), (
                ('Alexander', 'contenttypes'),
                ('Alexander', 'app'),
                ('Julia', 'contenttypes'),
                ('Simon', 'contenttypes')
            )
        )
        # Swap order
        f1_ordering.order, f2_ordering.order = f2_ordering.order, f1_ordering.order
        f1_ordering.save()
        f2_ordering.save()
        self.assertSequenceEqual(
            Model.objects.values_list('f1', 'f2__app_label'), (
                ('Alexander', 'contenttypes'),
                ('Julia', 'contenttypes'),
                ('Simon', 'contenttypes'),
                ('Alexander', 'app')
            )
        )
        # Swap the ordering to descending
        f1_ordering.descending = True
        f1_ordering.save()
        self.assertSequenceEqual(
            Model.objects.values_list('f1', 'f2__app_label'), (
                ('Simon', 'contenttypes'),
                ('Julia', 'contenttypes'),
                ('Alexander', 'contenttypes'),
                ('Alexander', 'app')
            )
        )
        f1_ordering.delete()
        f2_ordering.delete()


class UniqueTogetherDefinitionTest(BaseModelDefinitionTestCase):
    def setUp(self):
        super(UniqueTogetherDefinitionTest, self).setUp()
        self.f1 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f1', max_length=25)
        self.f2 = CharFieldDefinition.objects.create(model_def=self.model_def,
                                                     name='f2', max_length=25)
        self.ut = UniqueTogetherDefinition.objects.create(model_def=self.model_def)
        self.Model = self.model_def.model_class()

    def test_repr(self):
        """
        Make sure UniqueTogetherDefinition objects are always repr()-able.
        """
        repr(self.ut)
        repr(UniqueTogetherDefinition())

    def test_clean(self):
        """
        Make sure we can't create a unique key with two fields of two
        different models
        """
        other_model_def = ModelDefinition.objects.create(app_label='app',
                                                         object_name='OtherModel')
        f2 = CharFieldDefinition.objects.create(model_def=other_model_def,
                                                name='f2', max_length=25)
        self.ut.field_defs = (self.f1, f2)
        with self.assertRaises(ValidationError):
            self.ut.clean()
        other_model_def.delete()

    def test_db_column(self):
        """
        Make sure a unique index creation works correctly when using a custom
        `db_column`. This is needed for unique FK's columns.
        """
        self.f2.db_column = 'f2_column'
        self.f2.save()
        self.ut.field_defs = (self.f1, self.f2)
        self.f2.db_column = 'f2'
        self.f2.save()
        self.ut.delete()

    def test_cannot_create_unique(self):
        """
        Creating a unique key on a table with duplicate
        rows shouldn't work
        """
        self.Model.objects.create(f1='a', f2='b')
        self.Model.objects.create(f1='a', f2='b')
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.ut.field_defs = (self.f1, self.f2)

    def test_cannot_insert_duplicate_row(self):
        """
        Inserting a duplicate rows shouldn't work
        """
        self.Model.objects.create(f1='a', f2='b')
        self.ut.field_defs = (self.f1, self.f2)
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.Model.objects.create(f1='a', f2='b')

    def test_cannot_remove_unique(self):
        """
        Removing a unique constraint that cause
        duplicate rows shouldn't work
        """
        self.ut.field_defs = (self.f1, self.f2)
        self.Model.objects.create(f1='a', f2='b')
        self.Model.objects.create(f1='a', f2='c')
        with captured_stderr():
            with self.assertRaises(IntegrityError):
                self.ut.field_defs.remove(self.f2)

    def test_clear_removes_unique(self):
        """
        Removing a unique constraint should relax duplicate row
        validation
        """
        self.Model.objects.create(f1='a', f2='b')
        self.ut.field_defs = self.f1, self.f2
        self.ut.field_defs.clear()
        self.Model.objects.create(f1='a', f2='b')


class BaseDefinitionTest(BaseModelDefinitionTestCase):
    def test_clean(self):
        """
        Ensure `BaseDefinition.clean` works correctly.
        """
        bd = BaseDefinition(model_def=self.model_def)
        # Base must be a class
        bd.base = BaseDefinitionTest.test_clean
        self.assertRaisesMessage(ValidationError,
                                 _('Base must be a class.'), bd.clean)
        # Subclasses of MutableModel are valid bases
        bd.base = ModelDefinition.objects.create(app_label='app',
                                                 object_name='AnotherModel').model_class()
        try:
            bd.clean()
        except ValidationError:
            self.fail('MutableModel subclasses are valid bases.')
        # But model definition can't be bases of themselves
        bd.base = self.model_def.model_class()
        self.assertRaisesMessage(ValidationError,
                                 _("A model definition can't be a base of "
                                   "itself."), bd.clean)
        # Mixin objets are valid bases
        bd.base = Mixin
        try:
            bd.clean()
        except ValidationError:
            self.fail('Mixin objets are valid bases.')
        # Abstract model subclasses are valid bases
        bd.base = AbstractModel
        try:
            bd.clean()
        except ValidationError:
            self.fail('Abstract Model are valid bases')
        # Proxy model are not valid bases
        bd.base = ProxyModel
        self.assertRaisesMessage(ValidationError,
                                 _("Base can't be a proxy model."), bd.clean)

    @skipUnlessMutantModelDBFeature('supports_joins')
    def test_mutable_model_base(self):
        another_model_def = ModelDefinition.objects.create(app_label='app',
                                                           object_name='AnotherModel')
        AnotherModel = another_model_def.model_class()
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name='f1', max_length=25)
        base_definition = BaseDefinition(model_def=another_model_def)
        base_definition.base = self.model_def.model_class()
        base_definition.save()
        another_model = AnotherModel.objects.create(f1='Martinal')
        self.assertTrue(AnotherModel.objects.exists())
        CharFieldDefinition.objects.create(model_def=self.model_def,
                                           name='f2', max_length=25, null=True)
        another_model = AnotherModel.objects.get(pk=another_model.pk)
        self.assertIsNone(another_model.f2)
        another_model.f2 = 'Placebo'
        another_model.save()

    def test_base_inheritance(self):
        Model = self.model_def.model_class()
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=Mixin)
        self.assertTrue(issubclass(Model, Mixin))
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=AbstractModel)
        self.assertTrue(issubclass(Model, Mixin) and
                        issubclass(Model, AbstractModel))

    def test_base_ordering(self):
        Model = self.model_def.model_class()
        BaseDefinition.objects.create(model_def=self.model_def,
                                      base=Mixin, order=2)
        model_subclass_def = BaseDefinition.objects.create(model_def=self.model_def,
                                                           base=AbstractModel,
                                                           order=1)
        instance = Model()
        self.assertEqual('AbstractModel', instance.method())
        model_subclass_def.order = 3
        model_subclass_def.save()
        instance = Model()
        self.assertEqual('Mixin', instance.method())

    def test_abstract_field_inherited(self):
        bd = BaseDefinition.objects.create(model_def=self.model_def,
                                           base=AbstractModel)
        Model = self.model_def.model_class()
        Model.objects.create(abstract_model_field='value')
        # Test column alteration and addition by replacing the base with
        # a new one with a field with the same name and a second field.
        bd.base = ModelSubclassWithTextField
        bd.save()
        Model.objects.get(abstract_model_field='value')
        # The original CharField should be replaced by a TextField with no
        # max_length and a second field should be added
        Model.objects.create(abstract_model_field='another one bites the dust',
                             second_field=True)
        # Test column deletion by deleting the base
        # This should cause the model to loose all it's fields and the table
        # to loose all it's columns
        bd.delete()
        self.assertEqual(list(Model.objects.values_list()),
                         [(instance.id,) for instance in Model.objects.all()])
        self.assertModelTablesColumnDoesntExists(Model, 'field')



from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.db.models import fields
from django.db.models.signals import class_prepared

from ...models import ModelDefinition
from django.db.models.fields import FieldDoesNotExist


class ModelClassAttributeDescriptor(object):
    """
    Provide an access to an attribute of  a model definition's underlying
    model class. Useful for defining an accessor to a manager.
    """
    def __init__(self, model_def_name, attr_name):
        self.model_def_name = model_def_name
        self.attr_name = attr_name

    def __validate(self, **kwargs):
        opts = self.model._meta
        try:
            field = opts.get_field(self.model_def_name)
        except FieldDoesNotExist:
            raise ImproperlyConfigured("%s.%s.%s refers to an inexistent field "
                                       "'%s'" % (opts.app_label, opts.object_name,
                                                 self.name, self.model_def_name))
        else:
            if (not isinstance(field, fields.related.ForeignKey) or
                not issubclass(field.rel.to, ModelDefinition)):
                raise ImproperlyConfigured("%s.%s.%s must refer to a ForeignKey "
                                           "to `ModelDefinition`"
                                           % (opts.app_label, opts.object_name,
                                              self.name))
        setattr(self.model, self.name, self)

    def contribute_to_class(self, cls, name):
        self.model = cls
        self.name = name
        class_prepared.connect(self.__validate, cls, weak=True)

    def __get__(self, instance, instance_type=None):
        if instance:
            try:
                model_def = getattr(instance, self.model_def_name)
            except ModelDefinition.DoesNotExist:
                pass
            else:
                if model_def is not None:
                    return getattr(model_def.model_class(), self.attr_name)
            raise AttributeError("Can't access attribute '%s' of the "
                                 "model defined by '%s' since it doesn't exist."
                                 % (self.attr_name, self.model_def_name))
        else:
            return self

    def __set__(self, instance, value):
        raise AttributeError("Can't set attribute")



from __future__ import unicode_literals

from ...managers import FilteredQuerysetManager
from ...models import FieldDefinitionManager


class ForeignKeyDefinitionManager(FilteredQuerysetManager,
                                  FieldDefinitionManager):
    pass



from __future__ import unicode_literals

from django.db.models import fields
from django.utils.translation import ugettext_lazy as _

from ...models.field import FieldDefinition, FieldDefinitionManager


class _NumericMeta:
    defined_field_category = _('Numeric')


class SmallIntegerFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.SmallIntegerField


class PositiveSmallIntegerFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.PositiveSmallIntegerField


class IntegerFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.IntegerField


class PositiveIntegerFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.PositiveIntegerField


class BigIntegerFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.BigIntegerField


class FloatFieldDefinition(FieldDefinition):
    class Meta(_NumericMeta):
        app_label = 'mutant'
        proxy = True
        defined_field_class = fields.FloatField


max_digits_help_text = _('The maximum number of digits allowed in the number. '
                         'Note that this number must be greater than or equal '
                         'to ``decimal_places``, if it exists.')
decimal_places_help_text = _('The number of decimal places to store '
                             'with the number.')

class DecimalFieldDefinition(FieldDefinition):
    max_digits = fields.PositiveSmallIntegerField(_('max digits'),
                                                  help_text=max_digits_help_text)
    decimal_places = fields.PositiveSmallIntegerField(_('decimal_places'),
                                                      help_text=decimal_places_help_text)

    objects = FieldDefinitionManager()

    class Meta(_NumericMeta):
        app_label = 'mutant'
        defined_field_class = fields.DecimalField
        defined_field_options = ('max_digits', 'decimal_places',)



from __future__ import unicode_literals

import django
from django.db.models.fields import TextField
from django.utils.encoding import smart_unicode
from django.utils.functional import Promise
from django.utils.translation import ugettext_lazy as _


if django.VERSION[0:2] > (1, 4):
    _delegate_bytes = '_delegate_bytes'
    _delegate_text = '_delegate_text'
else:
    _delegate_bytes = '_delegate_str'
    _delegate_text = '_delegate_unicode'

def _is_gettext_promise(value):
    return isinstance(value, Promise) and (getattr(value, _delegate_bytes) or
                                           getattr(value, _delegate_text))


class LazilyTranslatedField(TextField):
    def to_python(self, value):
        if value is None or _is_gettext_promise(value):
            return value
        return _(smart_unicode(value))

    def get_prep_value(self, value):
        if value is None:
            return value
        elif _is_gettext_promise(value):
            value = smart_unicode(value._proxy____args[0])
        return smart_unicode(value)



from __future__ import unicode_literals

import warnings

import django
from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


# TODO: Remove when support for Django 1.5 is dropped.
if django.VERSION < (1, 7):
    class ProxyAwareGenericForeignKey(GenericForeignKey):
        """
        Basically a GenericForeignKey that saves the actual ContentType of the
        object even if it's a proxy Model.
        """
        if django.VERSION >= (1, 6):
            def __init__(self, *args, **kwargs):
                warnings.warn(
                    '`ProxyAwareGenericForeignKey` is deprecated on Django >= '
                    '1.6. Use `GenericForeignKey` with the '
                    '`for_concrete_model=False` kwarg instead.',
                    DeprecationWarning, stacklevel=2
                )
                super(ProxyAwareGenericForeignKey, self).__init__(*args, **kwargs)

        def get_content_type(self, obj=None, **kwargs):
            if obj:
                return ContentType.objects.db_manager(obj._state.db).get_for_model(
                    obj.__class__, for_concrete_model=False
                )
            else:
                return super(ProxyAwareGenericForeignKey, self).get_content_type(obj, **kwargs)



from __future__ import unicode_literals

import sys
# TODO: Remove when support for Python 2.6 is dropped
if sys.version_info >= (2, 7):
    from unittest import skipIf
else:
    from django.utils.unittest import skipIf

from django.db import connection
from django.utils.translation import ugettext_lazy as _
import south

from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import BooleanFieldDefinition, NullBooleanFieldDefinition


class BooleanFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Boolean')

    @skipIf(
        connection.settings_dict['ENGINE'] == 'django.db.backends.sqlite3' and
        south.__version__ == '0.8.1',
        "South 0.8.1 doesn't escape added column default value correctly on SQLite3."
    )
    def test_create_with_default(self):
        super(BooleanFieldDefinitionTestMixin, self).test_create_with_default()


class BooleanFieldDefinitionTest(BooleanFieldDefinitionTestMixin,
                                 BaseModelDefinitionTestCase):
    field_definition_cls = BooleanFieldDefinition
    field_definition_init_kwargs = {'default': True}
    field_values = (True, False)


class NullBooleanFieldDefinitionTest(BooleanFieldDefinitionTestMixin,
                                     BaseModelDefinitionTestCase):
    field_definition_cls = NullBooleanFieldDefinition
    field_values = (True, None)



from __future__ import unicode_literals

from functools import wraps

import django
from django.contrib.contenttypes.models import ContentType
from django.db import models, router
from django.db.models.fields import FieldDoesNotExist
from django.db.models.signals import (m2m_changed, post_delete, post_save,
    pre_delete)
from django.dispatch.dispatcher import receiver
from south.db import dbs

from .. import logger
from ..models import (ModelDefinition, BaseDefinition, FieldDefinition,
    UniqueTogetherDefinition)
from ..utils import popattr


def allow_syncdbs(model):
    for alias, db in dbs.iteritems():
        if router.allow_syncdb(alias, model):
            yield db


def perform_ddl(model, action, *args, **kwargs):
    if model._meta.managed:
        return

    for db in allow_syncdbs(model):
        if db.deferred_sql:
            for statement in db.deferred_sql:
                logger.warn("Clearing non-executed deferred SQL statement "
                            "since we can't assume it's safe to execute it now. "
                            "Statements was: %s", statement)
            db.clear_deferred_sql()

        if django.VERSION >= (1, 6):
            getattr(db, action)(*args, **kwargs)
            db.execute_deferred_sql()
        else:
            db.start_transaction()
            try:
                getattr(db, action)(*args, **kwargs)
                db.execute_deferred_sql()
            except Exception:
                db.rollback_transaction()
                raise
            else:
                db.commit_transaction()


def nonraw_instance(receiver):
    """
    A signal receiver decorator that fetch the complete instance from db when
    it's passed as raw
    """
    @wraps(receiver)
    def wrapper(sender, instance, raw, using, **kwargs):
        if raw:
            instance = sender._default_manager.using(using).get(pk=instance.pk)
        return receiver(sender=sender, raw=raw, instance=instance, using=using,
                        **kwargs)
    return wrapper


@receiver(post_save, sender=ModelDefinition,
          dispatch_uid='mutant.management.model_definition_post_save')
@nonraw_instance
def model_definition_post_save(sender, instance, created, **kwargs):
    model_class = instance.model_class(force_create=True)
    opts = model_class._meta
    if created:
        primary_key = opts.pk
        fields = [(field.get_attname_column()[1], field) for field in opts.fields
                  if field is not primary_key]
        try:
            extra_fields = getattr(instance._state, '_create_extra_fields')
        except AttributeError:
            pass
        else:
            for column, field in extra_fields:
                if field.primary_key:
                    primary_key = field
                else:
                    fields.append((column, field))
            delattr(instance._state, '_create_extra_fields')
        fields.insert(0, (primary_key.get_attname_column()[1], primary_key))
        try:
            delayed_save = getattr(instance._state, '_create_delayed_save')
        except AttributeError:
            pass
        else:
            for obj in delayed_save:
                obj.model_def = instance
                obj.save(force_insert=True, force_create_model_class=False)
            delattr(instance._state, '_create_delayed_save')
        perform_ddl(model_class, 'create_table', opts.db_table, fields)
        instance.model_class(force_create=True)
    elif instance._model_class:
        old_opts = instance._model_class._meta
        if old_opts.db_table != opts.db_table:
            perform_ddl(model_class, 'rename_table', old_opts.db_table, opts.db_table)
            # It means that the natural key has changed
            ContentType.objects.clear_cache()


@receiver(pre_delete, sender=ModelDefinition,
          dispatch_uid='mutant.management.model_definition_pre_delete')
def model_definition_pre_delete(sender, instance, **kwargs):
    model_class = instance.model_class()
    instance._state._deletion = (
        model_class,
        model_class._meta.db_table
    )


@receiver(post_delete, sender=ModelDefinition,
          dispatch_uid='mutant.management.model_definition_post_delete')
def model_definition_post_delete(sender, instance, **kwargs):
    model_class, table_name = popattr(instance._state, '_deletion')
    perform_ddl(model_class, 'delete_table', table_name)


@receiver(post_save, sender=BaseDefinition,
          dispatch_uid='mutant.management.base_definition_post_save')
def base_definition_post_save(sender, instance, created, raw, **kwargs):
    declared_fields = instance.get_declared_fields()
    if declared_fields:
        model_class = instance.model_def.model_class()
        opts = model_class._meta
        table_name = opts.db_table
        if created:
            try:
                add_columns = getattr(instance._state, '_add_columns')
            except AttributeError:
                add_columns = True
            else:
                delattr(instance._state, '_add_columns')
            finally:
                if add_columns:
                    for field in declared_fields:
                        perform_ddl(model_class, 'add_column', table_name,
                                    field.name, field)
        else:
            for field in declared_fields:
                try:
                    old_field = opts.get_field(field.name)
                except FieldDoesNotExist:
                    perform_ddl(model_class, 'add_column', table_name,
                                field.name, field)
                else:
                    column = old_field.get_attname_column()[1]
                    perform_ddl(model_class, 'alter_column', table_name,
                                column, field)


@receiver(pre_delete, sender=BaseDefinition,
          dispatch_uid='mutant.management.base_definition_pre_delete')
def base_definition_pre_delete(sender, instance, **kwargs):
    """
    This is used to pass data required for deletion to the post_delete
    signal that is no more available thereafter.
    """
    # see CASCADE_MARK_ORIGIN's docstring
    if popattr(instance._state, '_cascade_deletion_origin', None) == 'model_def':
        return
    if (instance.base and issubclass(instance.base, models.Model) and
        instance.base._meta.abstract):
        model_class = instance.model_def.model_class()
        instance._state._deletion = (
            model_class,
            model_class._meta.db_table,
        )


@receiver(post_delete, sender=BaseDefinition,
          dispatch_uid='mutant.management.base_definition_post_delete')
def base_definition_post_delete(sender, instance, **kwargs):
    """
    Make sure to delete fields inherited from an abstract model base.
    """
    if hasattr(instance._state, '_deletion'):
        model, table_name = popattr(instance._state, '_deletion')
        for field in instance.base._meta.fields:
            perform_ddl(model, 'delete_column', table_name, field.name)


@receiver(m2m_changed, sender=UniqueTogetherDefinition.field_defs.through,
          dispatch_uid='mutant.management.unique_together_field_defs_changed')
def unique_together_field_defs_changed(instance, action, model, **kwargs):
    # Here we can't use kwargs['pk_set'] since we need a reference to columns
    # *before* they're actually saved for unique deletion.
    # Note that we use a list and not a tuple for columns since SQLite
    # `delete_unique` chokes if we don't.
    columns = [
        field_def._south_ready_field_instance().get_attname_column()[1]
        for field_def in instance.field_defs.select_subclasses()
    ]
    # If there's no columns and action is post_clear there's nothing to do
    if columns and action != 'post_clear':
        model_class = instance.model_def.model_class()
        table_name = model_class._meta.db_table
        if action in ('pre_add', 'pre_remove', 'pre_clear'):
            perform_ddl(model_class, 'delete_unique', table_name, columns)
        # Safe guard against m2m_changed.action API change
        elif action in ('post_add', 'post_remove'):
            perform_ddl(model_class, 'create_unique', table_name, columns)


@receiver(post_save, sender=FieldDefinition,
          dispatch_uid='mutant.management.raw_field_definition_proxy_post_save')
def raw_field_definition_proxy_post_save(sender, instance, raw, **kwargs):
    """
    When proxy field definitions are loaded from a fixture they're not
    passing through the `field_definition_post_save` signal. Make sure they
    are.
    """
    if raw:
        model_class = instance.content_type.model_class()
        opts = model_class._meta
        if opts.proxy and opts.concrete_model is sender:
            field_definition_post_save(
                sender=model_class, instance=instance.type_cast(), raw=raw,
                **kwargs
            )


@nonraw_instance
def field_definition_post_save(sender, instance, created, raw, **kwargs):
    """
    This signal is connected by all FieldDefinition subclasses
    see comment in FieldDefinitionBase for more details
    """
    # If the field definition is raw we must re-create the model class
    # since ModelDefinitionAttribute.save won't be called
    model_class = instance.model_def.model_class(force_create=raw)
    table_name = model_class._meta.db_table
    field = instance._south_ready_field_instance()
    if created:
        if hasattr(instance._state, '_creation_default_value'):
            field.default = instance._state._creation_default_value
            delattr(instance._state, '_creation_default_value')
        try:
            add_column = getattr(instance._state, '_add_column')
        except AttributeError:
            add_column = True
        else:
            delattr(instance._state, '_add_column')
        finally:
            if add_column:
                perform_ddl(model_class, 'add_column', table_name,
                            instance.name, field)
    else:
        column = field.get_attname_column()[1]
        old_field = instance._state._pre_save_field
        delattr(instance._state, '_pre_save_field')
        # Field renaming
        old_column = old_field.get_attname_column()[1]
        if column != old_column:
            perform_ddl(model_class, 'rename_column', table_name, old_column, column)
        # Create/Drop unique and primary key
        for opt in ('primary_key', 'unique'):
            value = getattr(field, opt)
            if value != getattr(old_field, opt):
                action_prefix = 'create' if value else 'delete'
                action = "%s_%s" % (action_prefix, opt)
                perform_ddl(model_class, action, table_name, (column,))
        perform_ddl(model_class, 'alter_column', table_name, column, field)

FIELD_DEFINITION_POST_SAVE_UID = "mutant.management.%s_post_save"


@receiver(pre_delete, sender=FieldDefinition,
          dispatch_uid='mutant.management.field_definition_pre_delete')
def field_definition_pre_delete(sender, instance, **kwargs):
    # see CASCADE_MARK_ORIGIN's docstring
    if popattr(instance._state, '_cascade_deletion_origin', None) == 'model_def':
        return
    model_class = instance.model_def.model_class()
    opts = model_class._meta
    field = opts.get_field(instance.name)
    instance._state._deletion = (
        model_class,
        opts.db_table,
        field,
    )


@receiver(post_delete, sender=FieldDefinition,
          dispatch_uid='mutant.management.field_definition_post_delete')
def field_definition_post_delete(sender, instance, **kwargs):
    if hasattr(instance._state, '_deletion'):
        model, table_name, field = popattr(instance._state, '_deletion')
        column = field.get_attname_column()[1]
        if field.primary_key:
            primary_key = models.AutoField(name='id', primary_key=True)
            perform_ddl(model, 'alter_column', table_name, column, primary_key)
        else:
            perform_ddl(model, 'delete_column', table_name, column)



from __future__ import unicode_literals

from django.core.signals import request_started
from django.db import reset_queries


try:
    from django.test.utils import CaptureQueriesContext
except ImportError:
    # TODO: Remove when support for Django < 1.6 is dropped
    class CaptureQueriesContext(object):
        """
        Context manager that captures queries executed by the specified connection.
        """
        def __init__(self, connection):
            self.connection = connection

        def __iter__(self):
            return iter(self.captured_queries)

        def __getitem__(self, index):
            return self.captured_queries[index]

        def __len__(self):
            return len(self.captured_queries)

        @property
        def captured_queries(self):
            return self.connection.queries[self.initial_queries:self.final_queries]

        def __enter__(self):
            self.use_debug_cursor = self.connection.use_debug_cursor
            self.connection.use_debug_cursor = True
            self.initial_queries = len(self.connection.queries)
            self.final_queries = None
            request_started.disconnect(reset_queries)
            return self

        def __exit__(self, exc_type, exc_value, traceback):
            self.connection.use_debug_cursor = self.use_debug_cursor
            request_started.connect(reset_queries)
            if exc_type is not None:
                return
            self.final_queries = len(self.connection.queries)



#!/usr/bin/env python
import os
import re
from setuptools import find_packages, setup

from mutant import __version__


MODULE_PATH = os.path.abspath(os.path.dirname(__file__))

LINK_REQUIREMENT = re.compile(
    r'^https://.+#egg=(?P<package>.+)-(?P<version>\d(?:\.\d)*)$'
)


install_requires = ['django>=1.5']
dependency_links = []

for requirement in (l.strip() for l in open(os.path.join(MODULE_PATH, 'requirements/base.txt'))):
    match = LINK_REQUIREMENT.match(requirement)
    if match:
        install_requires.append("%(package)s==%(version)s" % match.groupdict())
        dependency_links.append(match.group())
    else:
        install_requires.append(requirement)


setup(
    name='django-mutant',
    version='.'.join(str(v) for v in __version__),
    description='Dynamic model definition and alteration (evolving schemas)',
    long_description=open(os.path.join(MODULE_PATH, 'README.md')).read(),
    url='https://github.com/charettes/django-mutant',
    author='Simon Charette',
    author_email='charette.s@gmail.com',
    install_requires=install_requires,
    dependency_links=dependency_links,
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    license='MIT License',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



from __future__ import unicode_literals

from decimal import Decimal

from django.utils.translation import ugettext_lazy as _

from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import (BigIntegerFieldDefinition, DecimalFieldDefinition,
    FloatFieldDefinition, IntegerFieldDefinition,
    PositiveIntegerFieldDefinition, PositiveSmallIntegerFieldDefinition,
    SmallIntegerFieldDefinition)


class NumericFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Numeric')


class SmallIntegerFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                      BaseModelDefinitionTestCase):
    field_definition_cls = SmallIntegerFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (-134, 245)


class PositiveSmallIntegerFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                              BaseModelDefinitionTestCase):
    field_definition_cls = PositiveSmallIntegerFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (135, 346)


class IntegerFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                 BaseModelDefinitionTestCase):
    field_definition_cls = IntegerFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (44323423, -4223423)


class PositiveIntegerFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                         BaseModelDefinitionTestCase):
    field_definition_cls = PositiveIntegerFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (44323423, 443234234)


class BigIntegerFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                    BaseModelDefinitionTestCase):
    field_definition_cls = BigIntegerFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (443234234324, 443234234998)


class FloatFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                               BaseModelDefinitionTestCase):
    field_definition_cls = FloatFieldDefinition
    field_definition_init_kwargs = {'default': 0}
    field_values = (1234567.84950, 18360935.1854195)


class DecimalFieldDefinitionTest(NumericFieldDefinitionTestMixin,
                                 BaseModelDefinitionTestCase):
    field_definition_cls = DecimalFieldDefinition
    field_definition_init_kwargs = {
        'default': 0,
        'max_digits': 15,
        'decimal_places': 7
    }
    field_values = (
        Decimal('1234567.84950'),
        Decimal('18360935.1854195'),
    )



from __future__ import unicode_literals


SECRET_KEY = 'secret'

INSTALLED_APPS = [
    'django.contrib.auth', # This is needed because of django bug
    'django.contrib.contenttypes',
    'south',
    'polymodels',
    'mutant',
    'mutant.contrib.boolean',
    'mutant.contrib.temporal',
    'mutant.contrib.file',
    'mutant.contrib.numeric',
    'mutant.contrib.text',
    'mutant.contrib.web',
    'mutant.contrib.related',
]

SKIP_SOUTH_TESTS = False

try:
    import django_coverage
except ImportError:
    pass
else:
    INSTALLED_APPS.append('django_coverage')


TEST_RUNNER = 'mutant.tests.runners.MutantTestSuiteRunner'

TEST_DEFAULT_LABELS = [
    'mutant',
    'boolean',
    'temporal',
    'file',
    'numeric',
    'related',
    'text',
    'web',
    'south',
    'polymodels',
]

COVERAGE_MODULE_EXCLUDES = [
    '^(?!mutant)',
    'tests$',
    'locale$',
    'fixtures$',
    'hacks$',
    'mutant.contrib.geo',
    'mutant.contrib.nonrel',
]


from __future__ import unicode_literals

import warnings

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _
from orderable.models import OrderableModel
from picklefield.fields import dbsafe_encode, PickledObjectField
from polymodels.fields import PolymorphicTypeField
from polymodels.models import BasePolymorphicModel
from polymodels.utils import copy_fields

from .managers import FieldDefinitionManager, FieldDefinitionChoiceManager
from ..model import ModelDefinitionAttribute
from ...db.fields import LazilyTranslatedField, PythonIdentifierField
from ...hacks import patch_model_option_verbose_name_raw
from ...utils import lazy_string_format, model_name, popattr


patch_model_option_verbose_name_raw()


NOT_PROVIDED = dbsafe_encode(models.NOT_PROVIDED)


class FieldDefinitionBase(models.base.ModelBase):
    FIELD_CLASS_ATTR = 'defined_field_class'
    FIELD_OPTIONS_ATTR = 'defined_field_options'
    FIELD_DESCRIPTION_ATTR = 'defined_field_description'
    FIELD_CATEGORY_ATTR = 'defined_field_category'

    DEFAULT_VERBOSE_NAME = _("%s field definition")
    DEFAULT_VERBOSE_NAME_PLURAL = _("%s field definitions")

    _base_definition = None
    _field_definitions = {}
    _proxies = {}
    _lookups = {}

    def __new__(cls, name, parents, attrs):
        if 'Meta' in attrs:
            Meta = attrs['Meta']

            field_description = popattr(Meta, cls.FIELD_DESCRIPTION_ATTR, None)

            field_class = popattr(Meta, cls.FIELD_CLASS_ATTR, None)
            if field_class:
                if not issubclass(field_class, models.Field):
                    msg = ("Meta's defined_field_class must be a subclass of "
                           "django.db.models.fields.Field")
                    raise ImproperlyConfigured(msg)
                elif field_description is None:
                    field_description = getattr(field_class, 'description', None)

            field_options = popattr(Meta, cls.FIELD_OPTIONS_ATTR, ())
            if field_options:
                if not isinstance(field_options, tuple):
                    msg = "Meta's defined_field_options must be a tuple"
                    raise ImproperlyConfigured(msg)

            field_category = popattr(Meta, cls.FIELD_CATEGORY_ATTR, None)

            has_verbose_name = hasattr(Meta, 'verbose_name')
            has_verbose_name_plural = hasattr(Meta, 'verbose_name_plural')
        else:
            field_class = None
            field_options = ()
            field_description = None
            field_category = None
            has_verbose_name = False
            has_verbose_name_plural = False

        definition = super(FieldDefinitionBase, cls).__new__(cls, name, parents, attrs)
        opts = definition._meta

        # Store the FieldDefinition cls
        if cls._base_definition is None:
            cls._base_definition = definition
        else:
            base_definition = cls._base_definition
            parents = [definition]
            while parents:
                parent = parents.pop(0)
                if isinstance(parent, cls):
                    parent_opts = parent._meta
                    if field_description is None:
                        field_description = getattr(parent_opts, cls.FIELD_DESCRIPTION_ATTR, None)
                    if field_class is None:
                        field_class = getattr(parent_opts, cls.FIELD_CLASS_ATTR, None)
                        if field_class and field_description is None:
                            field_description = field_class.description
                    field_options += getattr(parent_opts, cls.FIELD_OPTIONS_ATTR, ())
                    if field_category is None:
                        field_category = getattr(parent_opts, cls.FIELD_CATEGORY_ATTR, None)
                    if parent is not base_definition:
                        parents = list(parent.__bases__) + parents  # mimic mro

            from ...management import (field_definition_post_save,
                FIELD_DEFINITION_POST_SAVE_UID)
            post_save_dispatch_uid = FIELD_DEFINITION_POST_SAVE_UID % model_name(definition._meta)
            signals.post_save.connect(field_definition_post_save, definition,
                                      dispatch_uid=post_save_dispatch_uid)

            # Warn the user that they should rely on signals instead of
            # overriding the delete methods since it might not be called
            # when deleting the associated model definition.
            if definition.delete != base_definition.delete:
                concrete_model = opts.concrete_model
                if (opts.proxy and
                    concrete_model.delete != base_definition.delete):
                    # Because of the workaround for django #18083 in
                    # FieldDefinition, overriding the `delete` method on a proxy
                    # of a concrete FieldDefinition that also override the
                    # delete method might call some deletion code twice.
                    # Until #18083 is fixed and the workaround is removed we
                    # raise a `TypeError` to prevent this from happening.
                    msg = ("Proxy model deletion is partially broken until "
                           "django #18083 is fixed. To work around this issue, "
                           "mutant make sure to call the concrete `FieldDefinition`"
                           "you are proxying, in this case `%(concrete_cls)s`. "
                           "However, this can trigger a double execution of "
                           "`%(concrete_cls)s.delete`, thus it is prohibited.")
                    raise TypeError(msg % {'concrete_cls': concrete_model.__name__})
                def_name = definition.__name__
                warnings.warn("Avoid overriding the `delete` method on "
                              "`FieldDefinition` subclass `%s` since it won't "
                              "be called when the associated `ModelDefinition` "
                              "is deleted. If you want to perform actions on "
                              "deletion, add hooks to the `pre_delete` and "
                              "`post_delete` signals." % def_name, UserWarning)

        setattr(definition._meta, cls.FIELD_CLASS_ATTR, field_class)
        setattr(definition._meta, cls.FIELD_OPTIONS_ATTR, tuple(set(field_options)))
        setattr(definition._meta, cls.FIELD_DESCRIPTION_ATTR, field_description)
        setattr(definition._meta, cls.FIELD_CATEGORY_ATTR, field_category)

        if field_description is not None:
            if not has_verbose_name:
                verbose_name = lazy_string_format(cls.DEFAULT_VERBOSE_NAME, field_description)
                definition._meta.verbose_name = verbose_name
                if not has_verbose_name_plural:
                    verbose_name_plural = lazy_string_format(cls.DEFAULT_VERBOSE_NAME_PLURAL, field_description)
                    definition._meta.verbose_name_plural = verbose_name_plural

        if field_class is not None:
            cls._field_definitions[field_class] = definition

        return definition


class FieldDefinition(BasePolymorphicModel, ModelDefinitionAttribute):
    __metaclass__ = FieldDefinitionBase

    FIELD_DEFINITION_PK_ATTR = '_mutant_field_definition_pk'

    CONTENT_TYPE_FIELD = 'content_type'
    content_type = PolymorphicTypeField('self')

    name = PythonIdentifierField(_('name'))
    verbose_name = LazilyTranslatedField(_('verbose name'), blank=True, null=True)
    help_text = LazilyTranslatedField(_('help text'), blank=True, null=True)

    null = models.BooleanField(_('null'), default=False)
    blank = models.BooleanField(_('blank'), default=False)

    db_column = models.SlugField(_('db column'), max_length=30, blank=True, null=True)
    db_index = models.BooleanField(_('db index'), default=False)

    editable = models.BooleanField(_('editable'), default=True)
    default = PickledObjectField(_('default'), null=True, default=NOT_PROVIDED)

    primary_key = models.BooleanField(_('primary key'), default=False)
    unique = models.BooleanField(_('unique'), default=False)

    unique_for_date = PythonIdentifierField(_('unique for date'), blank=True, null=True)
    unique_for_month = PythonIdentifierField(_('unique for month'), blank=True, null=True)
    unique_for_year = PythonIdentifierField(_('unique for year'), blank=True, null=True)

    objects = FieldDefinitionManager()

    class Meta:
        app_label = 'mutant'
        verbose_name = _('field')
        verbose_name_plural = _('fields')
        unique_together = (('model_def', 'name'),)
        defined_field_options = (
            'name', 'verbose_name', 'help_text',
            'null', 'blank', 'db_column', 'db_index',
            'editable', 'default', 'primary_key', 'unique',
            'unique_for_date', 'unique_for_month', 'unique_for_year'
        )

    def natural_key(self):
        return self.model_def.natural_key() + (self.name,)
    natural_key.dependencies = ('mutant.modeldefinition',)

    def save(self, *args, **kwargs):
        if self.pk:
            self._state._pre_save_field = self.get_bound_field()
        return super(FieldDefinition, self).save(*args, **kwargs)

    def delete(self, *args, **kwargs):
        opts = self._meta
        if opts.proxy:
            # TODO: #18083
            # Ok so this is a big issue: proxy model deletion is completely
            # broken. When you delete a inherited model proxy only the proxied
            # model is deleted, plus deletion signals are not sent for the
            # proxied model and it's subclasses. Here we attempt to fix this by
            # getting the concrete model instance of the proxy and deleting it
            # while sending proxy model signals.
            concrete_model = opts.concrete_model
            concrete_model_instance = copy_fields(self, concrete_model)

            # Send proxy pre_delete
            signals.pre_delete.send(self.__class__, instance=self)

            # Delete the concrete model
            delete = concrete_model_instance.delete(*args, **kwargs)

            # This should be sent before the subclasses post_delete but we
            # cannot venture into deletion.Collector to much. Better wait until
            # #18083 is fixed.
            signals.post_delete.send(self.__class__, instance=self)

            return delete
        return super(FieldDefinition, self).delete(*args, **kwargs)

    def clone(self):
        options = dict((name, getattr(self, name))
                       for name in self.get_field_option_names())
        return self.__class__(**options)

    @classmethod
    def get_field_class(cls):
        field_class = getattr(cls._meta, FieldDefinitionBase.FIELD_CLASS_ATTR)
        if not field_class:
            raise NotImplementedError(
                "%s didn't define any `field_class`." % cls.__name__
            )
        return field_class

    @classmethod
    def get_field_option_names(cls):
        return getattr(cls._meta, FieldDefinitionBase.FIELD_OPTIONS_ATTR)

    @classmethod
    def get_field_description(cls):
        return getattr(cls._meta, FieldDefinitionBase.FIELD_DESCRIPTION_ATTR)

    @classmethod
    def get_field_category(cls):
        return getattr(cls._meta, FieldDefinitionBase.FIELD_CATEGORY_ATTR)

    @classmethod
    def get_content_type(cls):
        return ContentType.objects.get_for_model(cls, for_concrete_model=False)

    def get_field_choices(self):
        return tuple(self.choices.as_choices())

    def get_field_options(self, **overrides):
        model_opts = self._meta
        options = {}
        for name in self.get_field_option_names():
            if name in overrides:  # Avoid fetching if it's overridden
                continue
            value = getattr(self, name)
            if value != model_opts.get_field(name).get_default():
                options[name] = value
        if 'choices' not in overrides:  # Avoid fetching if it's overridden
            choices = self.get_field_choices()
            if choices:
                options['choices'] = choices
        return options

    def field_instance(self, **overrides):
        cls = self.get_field_class()
        options = self.get_field_options(**overrides)
        options.update(overrides)
        instance = cls(**options)
        setattr(instance, self.FIELD_DEFINITION_PK_ATTR, self.pk)
        return instance

    def get_bound_field(self):
        opts = self.model_def.model_class()._meta
        for field in opts.fields:
            if getattr(field, self.FIELD_DEFINITION_PK_ATTR, None) == self.pk:
                return field

    def _south_ready_field_instance(self):
        """
        South api sometimes needs to have modified version of fields to work.
        i. e. You can't pass a ForeignKey(to='self') to add_column
        """
        return self.field_instance()

    def clean(self):
        # Make sure we can build the field
        try:
            field = self.field_instance()
        except NotImplementedError:
            pass  # `get_field_class` is not implemented
        except Exception as e:
            raise ValidationError(e)
        else:
            # Test the specified default value
            if field.has_default():
                default = field.get_default()
                try:
                    field.clean(default, None)
                except Exception:
                    msg = _("%r is not a valid default value") % default
                    raise ValidationError({'default': [msg]})


class FieldDefinitionChoice(OrderableModel):
    """
    A Model to allow specifying choices for a field definition instance
    """
    field_def = models.ForeignKey(FieldDefinition, related_name='choices')
    group = LazilyTranslatedField(_('group'), blank=True, null=True)
    value = PickledObjectField(_('value'), editable=True)
    label = LazilyTranslatedField(_('label'))

    objects = FieldDefinitionChoiceManager()

    class Meta(OrderableModel.Meta):
        app_label = 'mutant'
        verbose_name = _('field definition choice')
        verbose_name_plural = _('field definition choices')
        unique_together = (
            ('field_def', 'order'),
            ('field_def', 'group', 'value')
        )

    def clean(self):
        try:
            # Make sure to create a field instance with no choices to avoid
            # validating against existing ones.
            field = self.field_def.type_cast().field_instance(choices=None)
            field.clean(self.value, None)
        except ValidationError as e:
            raise ValidationError({'value': e.messages})

    def save(self, *args, **kwargs):
        save = super(FieldDefinitionChoice, self).save(*args, **kwargs)
        self.field_def.model_def.model_class(force_create=True)
        return save



from __future__ import unicode_literals

from inspect import isclass
from itertools import chain

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError, ImproperlyConfigured
from django.db import models
from django.db.models.fields import FieldDoesNotExist
from django.db.models.signals import class_prepared
from django.utils.translation import ugettext_lazy as _
from orderable.models import OrderableModel
from picklefield.fields import PickledObjectField

try:
    from django.db.models.constants import LOOKUP_SEP
except ImportError:
    from django.db.models.sql.constants import LOOKUP_SEP

from ... import logger
from ...db.deletion import CASCADE_MARK_ORIGIN
from ...db.fields import LazilyTranslatedField, PythonIdentifierField
from ...db.models import MutableModel
from ...signals import mutable_class_prepared
from ...utils import get_db_table, model_name

from .managers import ModelDefinitionManager


def _model_class_from_pk(definition_cls, definition_pk):
    """
    Helper used to unpickle MutableModel model class from their definition
    pk.
    """
    try:
        return definition_cls.objects.get(pk=definition_pk).model_class()
    except definition_cls.DoesNotExist:
        pass


class _ModelClassProxy(object):
    def __init__(self, model_class):
        self.__dict__['model_class'] = model_class

    @property
    def __model_class_is_obsolete(self):
        return self.model_class.is_obsolete()

    @staticmethod
    def __get_underlying_model_class(value):
        if isinstance(value, _ModelClassProxy):
            return value.model_class
        elif isclass(value) and issubclass(value, MutableModel):
            return value

    def __get_model_class(self):
        if self.__model_class_is_obsolete:
            return self.__get__(None, None)
        else:
            return self.model_class

    def __get__(self, instance, owner):
        if self.__model_class_is_obsolete:
            try:
                definition = self.model_class.definition()
            except ModelDefinition.DoesNotExist:
                raise AttributeError('This model definition has been deleted')
            else:
                self.__dict__['model_class'] = definition.model_class()
        return self.model_class

    def __set__(self, instance, value):
        model_class = self.__get_underlying_model_class(value)
        if model_class is not None:
            self.model_class = model_class
        else:
            raise AttributeError('Invalid value')

    def __call__(self, *args, **kwargs):
        model_class = self.__get_model_class()
        return model_class(*args, **kwargs)

    def __getattr__(self, name):
        model_class = self.__get_model_class()
        return getattr(model_class, name)

    def __setattr__(self, name, value):
        model_class = self.__get_model_class()
        return setattr(model_class, name, value)

    def __delattr__(self, name):
        model_class = self.__get_model_class()
        return delattr(model_class, name)

    def __instancecheck__(self, instance):
        model_class = self.__get_model_class()
        return isinstance(instance, model_class)

    def __eq__(self, other):
        other_model_class = self.__get_underlying_model_class(other)
        if type(self.model_class) == type(other_model_class):
            return self.model_class == other_model_class
        else:
            return NotImplemented

    def __reduce__(self):
        model_class = self.__get_model_class()
        return (_model_class_from_pk, model_class._definition)

    def __str__(self):
        model_class = self.__get_model_class()
        return str(model_class)


class ModelDefinition(ContentType):
    object_name = PythonIdentifierField(_('object name'))
    db_table = models.CharField(_('database table'), max_length=63,
                                blank=True, null=True)
    managed = models.BooleanField(_('managed'), default=False)
    verbose_name = LazilyTranslatedField(_('verbose name'),
                                         blank=True, null=True)
    verbose_name_plural = LazilyTranslatedField(_('verbose name plural'),
                                                blank=True, null=True)

    objects = ModelDefinitionManager()

    class Meta:
        app_label = 'mutant'
        verbose_name = _('model definition')
        verbose_name_plural = _('model definitions')

    def __unicode__(self):
        return '.'.join((self.app_label, self.object_name))

    def natural_key(self):
        return (self.app_label, self.model)
    natural_key.dependencies = ('contenttypes.contenttype',)

    def __init__(self, *args, **kwargs):
        # Attach unsaved related objects
        bases = kwargs.pop('bases', ())
        fields = kwargs.pop('fields', ())
        extra_fields = []
        delayed_save = []
        for base in bases:
            assert base.pk is None, 'Cannot associate already existing BaseDefinition'
            extra_fields.extend(
                [(f.get_attname_column()[1], f) for f in base.get_declared_fields()]
            )
            base._state._add_columns = False
            delayed_save.append(base)
        for field in fields:
            assert field.pk is None, 'Cannot associate already existing FieldDefinition'
            field_instance = field._south_ready_field_instance()
            extra_fields.append(
                (field_instance.get_attname_column()[1], field_instance)
            )
            field._state._add_column = False
            delayed_save.append(field)
        super(ModelDefinition, self).__init__(*args, **kwargs)
        # Add those fields to the instance state to be retrieved later
        self._state._create_extra_fields = extra_fields
        self._state._create_delayed_save = delayed_save
        if self.pk:
            self._model_class = super(ModelDefinition, self).model_class()

    def get_model_bases(self):
        return tuple(bd.get_model_class() for bd in self.basedefinitions.all())

    def get_model_opts(self):
        attrs = {'app_label': self.app_label, 'managed': self.managed}
        # Database table
        db_table = self.db_table
        if db_table is None:
            db_table = get_db_table(*self.natural_key())
        attrs['db_table'] = db_table
        # Verbose names
        if self.verbose_name is not None:
            attrs['verbose_name'] = self.verbose_name
        if self.verbose_name_plural is not None:
            attrs['verbose_name_plural'] = self.verbose_name_plural
        # Unique togethers
        unique_together = tuple(tuple(utd.field_defs.names())
                                    for utd in self.uniquetogetherdefinitions.all())
        if unique_together:
            attrs['unique_together'] = unique_together
        # Ordering
        ordering = tuple(ordf.get_defined_ordering()
                            for ordf in self.orderingfielddefinitions.all())
        if ordering:
            # Make sure not to add ordering if it's empty since it would
            # prevent the model from inheriting it's possible base ordering.
            # Kinda related to django #17429
            attrs['ordering'] = ordering
        return type(str('Meta'), (), attrs)

    def get_model_attrs(self, existing_model_class=None):
        attrs = {
            'Meta': self.get_model_opts(),
            '__module__': str("mutant.apps.%s.models" % self.app_label),
            '_definition': (self.__class__, self.pk),
            '_is_obsolete': False,
            '_dependencies': set(),
        }
        attrs.update(dict((f.name, f.field_instance())
                            for f in self.fielddefinitions.select_subclasses()))
        return attrs

    def _create_model_class(self, existing_model_class=None):
        bases = self.get_model_bases()
        has_mutable_base = False
        for base in bases:
            if issubclass(base, MutableModel):
                has_mutable_base = True
                base._dependencies.add((self.__class__, self.pk))
        if not has_mutable_base:
            bases += (MutableModel,)
        attrs = self.get_model_attrs(existing_model_class)
        if existing_model_class:
            existing_model_class.mark_as_obsolete()
        model_class = type(str(self.object_name), bases, attrs)
        mutable_class_prepared.send(sender=model_class, definition=self,
                                    existing_model_class=existing_model_class)
        logger.debug("Created model class %s.", model_class)
        return model_class

    def model_class(self, force_create=False):
        model_class = super(ModelDefinition, self).model_class()
        if force_create or model_class is None:
            model_class = self._create_model_class(model_class)
        return _ModelClassProxy(model_class)

    @property
    def model_ct(self):
        try:
            content_type = self._contenttype_ptr_cache
        except AttributeError:
            content_type = ContentType.objects.get_for_id(
                self.contenttype_ptr_id
            )
            self._contenttype_ptr_cache = content_type
        return content_type

    def clean(self):
        """
        Ensure app_label doesn't override an installed app one
        since model collision could occur and would cause a lot of
        side effects, i. e.:

        Defining a new auth.User, while not tested, could override
        the existing one and create a beautiful mess in django's
        internals
        """
        try:
            models.loading.cache.get_app(self.app_label, emptyOK=True)
        except ImproperlyConfigured:
            pass
        else:
            raise ValidationError(_('Cannot cloak an installed app'))

    def save(self, *args, **kwargs):
        self.model = self.object_name.lower()
        save = super(ModelDefinition, self).save(*args, **kwargs)
        self._model_class = super(ModelDefinition, self).model_class()
        return save

    def delete(self, *args, **kwargs):
        model_class = self.model_class()
        delete = super(ModelDefinition, self).delete(*args, **kwargs)
        model_class.mark_as_obsolete()
        ContentType.objects.clear_cache()
        del self._model_class
        return delete


class ModelDefinitionAttribute(models.Model):
    """
    A mixin used to make sure models that alter the state of a defined model
    clear the cached version
    """
    model_def = models.ForeignKey(ModelDefinition, related_name="%(class)ss",
                                  on_delete=CASCADE_MARK_ORIGIN)

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        force_create = kwargs.pop('force_create_model_class', True)
        save = super(ModelDefinitionAttribute, self).save(*args, **kwargs)
        self.model_def.model_class(force_create=force_create)
        return save

    def delete(self, *args, **kwargs):
        force_create = kwargs.pop('force_create_model_class', True)
        delete = super(ModelDefinitionAttribute, self).delete(*args, **kwargs)
        self.model_def.model_class(force_create=force_create)
        return delete


class BaseDefinition(OrderableModel, ModelDefinitionAttribute):
    """
    Model used to represent bases of a ModelDefinition
    """
    base = PickledObjectField(_('base'))

    class Meta:
        app_label = 'mutant'
        ordering = ('order',)
        unique_together = (('model_def', 'order'),)

    def get_model_class(self):
        if isinstance(self.base, _ModelClassProxy):
            return self.base.__get__(None, None)
        return self.base

    def get_declared_fields(self):
        fields = []
        if issubclass(self.base, models.Model):
            opts = self.base._meta
            if opts.abstract:
                # Add fields inherited from base's abstract parent and
                # local fields.
                base_fields = chain(
                    opts.get_fields_with_model(),
                    opts.get_m2m_with_model()
                )
                for field, model in base_fields:
                    if model is None or model._meta.abstract:
                        fields.append(field)
            elif not opts.proxy:
                # This is a concrete model base, we must declare a o2o
                attr_name = '%s_ptr' % model_name(opts)
                fields.append(
                    models.OneToOneField(
                        self.base, name=attr_name, null=True,
                        auto_created=True, parent_link=True
                    )
                )
        return tuple(fields)

    def clean(self):
        try:
            if issubclass(self.base, models.Model):
                if self.base._meta.proxy:
                    raise ValidationError(_("Base can't be a proxy model."))
                elif (issubclass(self.base, MutableModel) and
                      self.base.definition() == self.model_def):
                    raise ValidationError(_("A model definition can't be a base of itself."))
        except TypeError:
            raise ValidationError(_('Base must be a class.'))
        return super(BaseDefinition, self).clean()


class OrderingFieldDefinition(OrderableModel, ModelDefinitionAttribute):
    lookup = models.CharField(max_length=255)
    descending = models.BooleanField(_('descending'), default=False)

    class Meta(OrderableModel.Meta):
        app_label = 'mutant'
        # TODO: Should be unique both it bugs order swapping
        # unique_together = (('model_def', 'order'),)

    def clean(self):
        """
        Make sure the lookup makes sense
        """
        if self.lookup == '?':  # Randomly sort
            return
        #TODO: Support order_with_respect_to...
        else:
            lookups = self.lookup.split(LOOKUP_SEP)
            opts = self.model_def.model_class()._meta
            valid = True
            while len(lookups):
                lookup = lookups.pop(0)
                try:
                    field = opts.get_field(lookup)
                except FieldDoesNotExist:
                    valid = False
                else:
                    if isinstance(field, models.ForeignKey):
                        opts = field.rel.to._meta
                    elif len(lookups):  # Cannot go any deeper
                        valid = False
                finally:
                    if not valid:
                        msg = _('This field doesn\'t exist')
                        raise ValidationError({'lookup': [msg]})

    def get_defined_ordering(self):
        return ("-%s" % self.lookup) if self.descending else self.lookup


class UniqueTogetherDefinition(ModelDefinitionAttribute):
    field_defs = models.ManyToManyField(
        'FieldDefinition', related_name='unique_together_defs'
    )

    class Meta:
        app_label = 'mutant'

    def __unicode__(self):
        if self.pk:
            names = ', '.join(self.field_defs.names())
            return _("Unique together of (%s)") % names
        return ''

    def clean(self):
        for field_def in self.field_defs.select_related('model_def'):
            if field_def.model_def != self.model_def:
                msg = _('All fields must be of the same model')
                raise ValidationError({'field_defs': [msg]})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import sys
# TODO: Remove when support for Python 2.6 is dropped
if sys.version_info >= (2, 7):
    from unittest import skipIf, skipUnless
else:
    from django.utils.unittest import skipIf, skipUnless

from django.db import connection
from django.db.utils import DatabaseError
from django.utils.translation import ugettext_lazy as _
import south

from mutant.test.testcases import FieldDefinitionTestMixin
from mutant.tests.utils import BaseModelDefinitionTestCase

from .models import CharFieldDefinition, TextFieldDefinition


class TextFieldDefinitionTestMixin(FieldDefinitionTestMixin):
    field_definition_category = _('Text')


class CharFieldDefinitionTest(TextFieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = CharFieldDefinition
    field_definition_init_kwargs = {'max_length': 255}
    field_values = ('Raptor Jesus', 'Nirvana')

    @skipUnless(
        connection.settings_dict['ENGINE'] != 'django.db.backends.sqlite3',
        "SQLite3 doesn't enforce CHAR length"
    )
    def test_field_max_length(self):
        self.field.max_length = 24
        self.field.save()
        Model = self.model_def.model_class()
        with self.assertRaises(DatabaseError):
            Model.objects.create(field='Simon' * 5)


class TextFieldDefinitionTest(TextFieldDefinitionTestMixin,
                              BaseModelDefinitionTestCase):
    field_definition_cls = TextFieldDefinition
    field_values = (
        """
        J'ai caché
        Mieux que partout ailleurs
        Au grand jardin de mon coeur
        Une petite fleur
        Cette fleur
        Plus jolie q'un bouquet
        Elle garde en secret
        Tous mes rêves d'enfant
        L'amour de mes parents
        Et tous ces clairs matins
        Fait d'heureux souvenirs lointains
        """,
        """
        Quand la vie
        Par moments me trahi
        Tu restes mon bonheur
        Petite fleur

        Sur mes vingt ans
        Je m'arrête un moment
        Pour respirer
        Le parfum que j'ai tant aimé

        Dans mon coeur
        Tu fleuriras toujours
        Au grand jardin d'amour
        Petite fleur
        """,
    )

    @skipIf(
        connection.settings_dict['ENGINE'] == 'django.db.backends.sqlite3' and
        south.__version__ == '0.8.1',
        "South 0.8.1 doesn't escape added column default value correctly on SQLite3."
    )
    def test_create_with_default(self):
        super(TextFieldDefinitionTest, self).test_create_with_default()



from __future__ import unicode_literals

from django.db import connections, router
from django.test.testcases import _deferredSkip

from mutant.db.models import MutableModel
from mutant.models.model import ModelDefinition
from mutant.test.testcases import ModelDefinitionDDLTestCase


def table_columns_iterator(db, table_name):
    connection = connections[db]
    cursor = connection.cursor()
    description = connection.introspection.get_table_description(cursor, table_name)
    return (row[0] for row in description)


def model_dbs(model):
    for db in connections:
        if router.allow_syncdb(db, model):
            yield db


def db_is_nonrel(db):
    """
    Returns True if the db is non-relational
    """
    try:
        from djangotoolbox.db.base import NonrelDatabaseWrapper
    except ImportError:
        return False
    else:
        return isinstance(connections[db], NonrelDatabaseWrapper)


class BaseModelDefinitionTestCase(ModelDefinitionDDLTestCase):
    def setUp(self):
        self.model_def = ModelDefinition.objects.create(
            app_label='app',
            object_name='Model'
        )

    def assertTableExists(self, db, table):
        tables = connections[db].introspection.table_names()
        msg = "Table '%s.%s' doesn't exist, existing tables are %s"
        self.assertTrue(table in tables, msg % (db, table, tables))

    def assertTableDoesntExists(self, db, table):
        self.assertRaises(AssertionError, self.assertTableExists, db, table)

    def assertModelTablesExist(self, model):
        table = model._meta.db_table
        for db in model_dbs(model):
            self.assertTableExists(db, table)

    def assertModelTablesDontExist(self, model):
        table = model._meta.db_table
        for db in model_dbs(model):
            self.assertTableDoesntExists(db, table)

    def assertColumnExists(self, db, table, column):
        if not db_is_nonrel(db):
            columns = tuple(table_columns_iterator(db, table))
            data = {
                'db': db,
                'table': table,
                'column': column,
                'columns': columns
            }
            self.assertIn(column, columns,
                          "Column '%(db)s.%(table)s.%(column)s' doesn't exist, "
                          "%(db)s.'%(table)s's columns are %(columns)s" % data)

    def assertColumnDoesntExists(self, db, table, column):
        if not db_is_nonrel(db):
            self.assertRaises(AssertionError, self.assertColumnExists,
                              db, table, column)

    def assertModelTablesColumnExists(self, model, column):
        table = model._meta.db_table
        for db in model_dbs(model):
            if not db_is_nonrel(db):
                self.assertColumnExists(db, table, column)

    def assertModelTablesColumnDoesntExists(self, model, column):
        table = model._meta.db_table
        for db in model_dbs(model):
            if not db_is_nonrel(db):
                self.assertColumnDoesntExists(db, table, column)


def skipIfMutantModelDBFeature(feature, default=False):
    dbs = tuple(model_dbs(MutableModel))

    def _dbs_have_feature():
        return all(getattr(connections[db].features, feature, default)
                   for db in dbs)
    return _deferredSkip(_dbs_have_feature,
                         "Databases %s have feature %s" % (dbs, feature))


def skipUnlessMutantModelDBFeature(feature, default=True):
    dbs = tuple(model_dbs(MutableModel))

    def _dbs_dont_have_feature():
        return all(not getattr(connections[db].features, feature, default)
                   for db in dbs)
    return _deferredSkip(_dbs_dont_have_feature,
                         "Databases %s don't have feature %s" % (dbs, feature))



from __future__ import unicode_literals

from django.core.management.commands.loaddata import Command
from django.core.serializers import python as python_serializer
from django.core.serializers.base import DeserializationError

from ...models import ModelDefinition


# Monkey patch `_get_model` to attempt loading a matching model definition
# when no existing model is found.
_python_serializer_get_model = python_serializer._get_model


def _get_model(model_identifier):
    try:
        return _python_serializer_get_model(model_identifier)
    except DeserializationError as e:
        try:
            model_def = ModelDefinition.objects.get_by_natural_key(
                *model_identifier.split('.')
            )
        except ModelDefinition.DoesNotExist:
            raise e
        return model_def.model_class()

python_serializer._get_model = _get_model
